__credits__ = (
"""
Copyright (C) 2007  Matthew Marshall

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
""")

__author__ = "Matthew Marshall <matthew@matthewmarshall.org>"


cdef extern from "stdlib.h":
    ctypedef unsigned int size_t
    cdef void *malloc(size_t size)
    cdef void free(void *ptr)
    cdef void *realloc(void *ptr, size_t size)

cdef extern from "GL/gl.h":
    ctypedef float GLfloat
    ctypedef float GLclampf
    ctypedef unsigned int GLenum
    ctypedef unsigned int GLbitfield
    ctypedef int GLint
    ctypedef unsigned int GLuint
    ctypedef int GLsizei
    ctypedef double GLdouble
    ctypedef double GLclampd
    ctypedef void GLvoid

    cdef void glPushClientAttrib(GLbitfield mask)
    cdef void glPopClientAttrib()
    cdef void glEnableClientState(GLenum array)
    cdef void glDisableClientState(GLenum array)

    cdef void glVertexPointer(GLint size, GLenum type, GLsizei stride,
            GLvoid *pointer)
    cdef void glTexCoordPointer(GLint size, GLenum type, GLsizei stride,
            GLvoid *pointer)
    cdef void glColorPointer(GLint size, GLenum type, GLsizei stride,
            GLvoid *pointer)

    cdef void glDrawArrays(GLenum mode, GLint first, GLsizei count)
    cdef void glDrawElements(GLenum mode, GLsizei count, GLenum type,
            GLvoid *indices)

    cdef void glEnable(GLenum cap)
    cdef void glBindTexture(GLenum target, GLuint texture)

    cdef int GL_SHORT
    cdef int GL_INT
    cdef int GL_FLOAT
    cdef int GL_UNSIGNED_BYTE
    cdef int GL_UNSIGNED_INT

    cdef int GL_COLOR_ARRAY
    cdef int GL_INDEX_ARRAY
    cdef int GL_TEXTURE_COORD_ARRAY
    cdef int GL_VERTEX_ARRAY

    cdef int GL_CLIENT_VERTEX_ARRAY_BIT
    cdef int GL_QUADS
    cdef int GL_QUAD_STRIP
    cdef int GL_TRIANGLES
    cdef int GL_TRIANGLE_FAN
    cdef int GL_TRIANGLE_STRIP
    cdef int GL_LINES
    cdef int GL_LINE_STRIP
    cdef int GL_LINE_LOOP
    cdef int GL_POINTS

    cdef int GL_TEXTURE_2D


_mode_mapping = {
    GL_QUADS:"QUADS",
    GL_QUAD_STRIP:"QUAD_STRIP",
    GL_TRIANGLES:"TRIANGLES",
    GL_TRIANGLE_STRIP:"TRIANGLE_STRIP",
    GL_TRIANGLE_FAN:"TRIANGLE_FAN",
    GL_LINES:"LINES",
    GL_LINE_STRIP:"LINE_STRIP",
    GL_LINE_LOOP:"LINE_LOOP",
    GL_POINTS:"POINTS",
}
for key in list(_mode_mapping):
    _mode_mapping[_mode_mapping[key]] = key


cdef struct vertexinfo_s:
    float x,y,u,v
    unsigned char red,green,blue,alpha

cdef class VertexArray:
    """
    ``VertexArray(mode="QUADS", use_colors=True)``

    OpenGL vertex arrays provide a way to push a large amount of data to the
    video hardware, very fast.  ``VertexArray`` provides an easy to use class
    to take advantage of this speed.

    ``VertexArray`` supports both item access and assignment using
    ``VertexInfo`` objects.  For example, this would work as expected:

        .. sourcecode:: python

            vertex_array[0].x += 1

    You can also assign using tuples, just like as documented in the append
    method:

        .. sourcecode:: python

            vertex_array[0] = (x, y, u, v, red, green, blue, alpha)
    """
    cdef vertexinfo_s * _vertex_info
    cdef int _len
    cdef int _allocated_size
    cdef int _use_colors

    cdef public object texture_id

    cdef int _mode

    def __init__(self, mode="QUADS", use_colors=True):
        self.mode = mode
        self._use_colors = use_colors
        self._len = 0
        self._allocated_size = 4
        self._vertex_info = <vertexinfo_s*>malloc(sizeof(vertexinfo_s)*4)
        self.texture_id = None

    def __dealloc__(self):
        if self._vertex_info is not NULL:
            free(self._vertex_info)

    property use_colors:
        """
        If ``True``, (the default,) the color data in the vertex array will be
        used. Otherwise, the previous color will be used for all vertexes.
        (such as set by ``rabbyt.set_gl_color()``.)
        """
        def __get__(self):
            return bool(self._use_colors)
        def __set__(self, c):
            self._use_colors = bool(c)

    property mode:
        """
        The default primitive drawing mode.

        Valid modes are "QUADS", "QUAD_STRIP", "TRIANGLES", "TRIANGLE_STRIP",
        "TRIANGLE_FAN", "LINES", "LINE_STRIP", "LINE_LOOP", or "POINTS".  These
        corrispond directly to the OpenGL privitive drawing modes.  (In fact,
        you can also use the OpenGL enumerations instead of the strings, if you
        prefer.)
        """
        def __get__(self):
            return self._mode
        def __set__(self, mode):
            if isinstance(mode, basestring):
                mode = _mode_mapping[mode]
            if mode not in _mode_mapping:
                raise ValueError("must be a valid mode")
            self._mode = mode

    def __getitem__(self, int index):
        if index >= self._len or index < 0:
            raise IndexError(str(index))
        return VertexInfo(self, index)

    def __setitem__(self, int index, item):
        if index >= self._len or index < 0:
            raise IndexError(str(index))
        if len(item) > 8:
            raise ValueError(
                    "Vertex information should be at most a length of 8")
        try:
            self._vertex_info[index].x = item[0]
            self._vertex_info[index].y = item[1]
            self._vertex_info[index].u = item[2]
            self._vertex_info[index].v = item[3]
            self._vertex_info[index].red = _to255(item[4])
            self._vertex_info[index].green = _to255(item[5])
            self._vertex_info[index].blue = _to255(item[6])
            self._vertex_info[index].alpha = _to255(item[7])
        except IndexError:
            pass

    cdef _grow(self, int additional_size, exact):
        cdef int new_size, i
        self._len = self._len + additional_size
        if exact:
            new_size = self._len
        else:
            new_size = self._allocated_size
            while self._len > new_size:
                new_size = new_size * 2
        if new_size > self._allocated_size:
            self._vertex_info = <vertexinfo_s*>realloc(self._vertex_info,
                    sizeof(vertexinfo_s)*new_size)
            # Give the data some defaults
            for i from self._allocated_size <= i < new_size:
                self._vertex_info[i].x = 0
                self._vertex_info[i].y = 0
                self._vertex_info[i].u = 0
                self._vertex_info[i].v = 0
                self._vertex_info[i].red = 255
                self._vertex_info[i].green = 255
                self._vertex_info[i].blue = 255
                self._vertex_info[i].alpha = 255
            self._allocated_size = new_size

    def set_size(self, int size):
        """
        ``set_size(size)``

        Sets the size explicitly.

        This is an alternative to using ``append`` and ``extend``.  (It should
        be a little faster.)
        """
        if size > self._len:
            self._grow(size-self._len, True)
        elif size < self._len:
            self._len = size
            self._allocated_size = size
            self._vertex_info = <vertexinfo_s*>realloc(self._vertex_info,
                    sizeof(vertexinfo_s)*size)

    def append(self, vertex):
        """
        ``append(vertex)``

        Appends a vertex to the array.

        vertex should be a tuple of the form
        ``(x,y,u,v,red,green,blue,alpha)``. (You can also make the tuple
        shorter; any missing values will be left at the default.)

        Note that ``vertex`` doesn't need to be a tuple, but anything
        supporting item access.  This includes ``VertexInfo`` instances.
        """
        self._grow(1, False)
        try:
            self[self._len - 1] = (0,0,0,0, 1,1,1,1)
            self[self._len - 1] = vertex
        except:
            # If there was an error assigning the item, we don't want to leave
            # dead vertex behind.
            self._len = self._len - 1
            raise

    def extend(self, vertexes):
        """
        ``extend(vertexes)``

        Iterates over vertexes, calling ``append(vertex)`` on each one.
        """
        for v in vertexes:
            self.append(v)

    def __len__(self):
        return self._len

    def enable_arrays(self):
        """
        ``enable_arrays()``

        Calls the ``glEnableClientState`` and ``gl*Pointer`` functions to set
        up this ``VertexArray`` for usage.  This is automatically called when
        you call ``render()``.

        If you use vertex arrays with PyOpenGL elsewhere in your program, you
        should always wrap this method in
        ``glPushClientAttrib``\/``glPopClientAttrib``.
        """
        glEnableClientState(GL_VERTEX_ARRAY)
        glVertexPointer(2, GL_FLOAT, sizeof(vertexinfo_s), self._vertex_info)
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)
        glTexCoordPointer(2, GL_FLOAT, sizeof(vertexinfo_s),
                &self._vertex_info[0].u)
        if self._use_colors:
            glEnableClientState(GL_COLOR_ARRAY)
            glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(vertexinfo_s),
                    &self._vertex_info[0].red)
        else:
            glDisableClientState(GL_COLOR_ARRAY)

    def render(self, start=0, end=None):
        """
        ``render([start,] [end])``

        Renders the array!

        ``start`` and ``end`` can be specified to only render part of the
        array.
        """
        if self._len == 0:
            return
        if self.texture_id is not None:
            if self.texture_id:
                glEnable(GL_TEXTURE_2D)
                glBindTexture(GL_TEXTURE_2D, self.texture_id)
            else:
                glDisable(GL_TEXTURE_2D)
        if end is None:
            end = self._len
        glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
        try:
            self.enable_arrays()
            glDrawArrays(self._mode, start, end-start)
        finally:
            glPopClientAttrib()


cdef class VertexArrayIndexes:
    """
    ``VertexArrayIndexes([sequence])``

    Provides a way to list by indexes the vertexes to be rendered.

    This is useful if you have multiple primitives sharing a vertex, and you
    want to either save memory or make it easier to modify the data.
    """
    cdef unsigned int * _indexes
    cdef unsigned int _len
    cdef unsigned int _allocated_size
    cdef unsigned int _mode

    def __init__(self, iterable=()):
        self._allocated_size = 0
        self._len = 0
        self._mode = 0

        self._grow(len(iterable), True)
        for i, index in enumerate(iterable):
            self._indexes[i] = index

    property mode:
        """
        If non zero, this mode will override the ``VertexArray``'s mode when
        render is called.
        """
        def __get__(self):
            return self._mode
        def __set__(self, mode):
            if not mode:
                mode = 0
            elif isinstance(mode, basestring):
                mode = _mode_mapping[mode]
            elif mode not in _mode_mapping:
                raise ValueError("must be a valid mode")
            self._mode = mode

    cdef _grow(self, int additional_size, exact):
        self._len = self._len + additional_size
        cdef int new_size, i
        if exact:
            new_size = self._len
        else:
            new_size = self._allocated_size
            if new_size == 0:
                new_size = 16
            while self._len > new_size:
                new_size = new_size * 2
        if new_size > self._allocated_size:
            self._indexes = <unsigned int *>realloc(self._indexes,
                    sizeof(unsigned int)*new_size)
            for i from self._allocated_size <= i < new_size:
                self._indexes[i] = 0
            self._allocated_size = new_size

    def append(self, unsigned int index):
        """
        ``append(index)``

        Appends an index to be drawn.
        """
        self._grow(1, False)
        self._indexes[self._len-1] = index

    def extend(self, indexes):
        for i in indexes:
            self.append(i)

    def __getitem__(self, unsigned int i):
        if i >= self._len:
            raise IndexError
        return self._indexes[i]
    def __setitem__(self, unsigned int i, unsigned int index):
        if i >= self._len:
            raise IndexError
        self._indexes[i] = index

    def render(self, VertexArray vertex_array not None, int start=0, end=None,
                enable_arrays=True, safety_check=True):
        """
        ``render(vertex_array, [start, end, enable_arrays, safety_check])``

        This renders the vertexes in ``vertex_array``, which should be an
        instance of ``VertexArray``.

        ``start`` and ``end`` define which vertexes to render, and default to
        rendering all of them.

        If ``enable_arrays`` is ``True``, (the default,)
        ``vertex_array.enable_arrays()`` will be called automatically, along
        with pushing and popping the client state.  If you are rendering many
        ``VertexArrayIndexes`` off of the same ``VertexArray``, you might see a
        small speedup by only calling ``enable_arrays()`` on the
        ``VertexArray`` once.

        If ``safety_check`` is ``True``, (the default,) the ``safety_check()``
        method will be called with the length of the ``VertexArray``.  This
        prevents segmentation faults, but adds a little overhead.  You can set
        this to ``False`` once your code is debugged.  (Just remember to look
        if you ever get a segfault!)
        """
        if end is None:
            end = self._len

        if end <= start:
            return

        if safety_check:
            self.safety_check(len(vertex_array))

        if self.mode:
            mode = self.mode
        else:
            mode = vertex_array.mode

        if enable_arrays:
            glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
        try:
            if enable_arrays:
                vertex_array.enable_arrays()
            glDrawElements(mode, end-start, GL_UNSIGNED_INT,
                    &self._indexes[start])
        finally:
            if enable_arrays:
                glPopClientAttrib()

    def safety_check(self, int max_index):
        """
        ``safety_check(max_index)``

        Loops through all of the indexes and raises ``ValueError`` if one is
        greater than or equal to ``max_index``.
        """
        cdef int i
        for i from 0 <= i < self._len:
            if self._indexes[i] >= max_index:
                raise ValueError("Index out of bounds")


cdef unsigned char _to255(float value):
    if value > 1.0:
        value = 1.0
    elif value < 0.0:
        value = 0.0
    return <unsigned char>(value * 255)

cdef class VertexInfo:
    """
    Provides an interface to a single vertex in a ``VertexArray`` object.

    You probably won't ever want to instance this class yourself.  Instead,
    use "``vertex_array[index]``\".

    There are a number of properties for modifing the vertex data:

        ``x``, ``y``, ``u``, ``v``, ``red``, ``green``, ``blue``, ``alpha``

    You can also use these shortcuts to read/write multiple values at a time:

        ``xy``, ``uv``, ``rgb``, ``rgba``

    For example::

        vertex_array[index].x += 5
        vertex_array[index].rgb = 1.0, 0.0, 0.0

    And just in case if it's not obvious, ``u`` and ``v`` are the texture
    coordinates.
    """
    cdef VertexArray _parent
    cdef unsigned int _index

    def __init__(self, VertexArray parent, unsigned int index):
        self._parent = parent
        self._index = index

    def check_index(self):
        """
        check_index()

        Checks to make sure that the stored index is valid.  ``IndexError`` is
        raised if it isn't.
        """
        if self._parent is None:
            raise RuntimeError("Accessing VertexInfo property without parent.")
        if self._index >= self._parent._len:
            raise IndexError

    property index:
        def __get__(self):
            return self._index

    property xy:
        def __get__(self):
            return (self.x, self.y)
        def __set__(self, xy):
            self.x, self.y = xy

    property uv:
        def __get__(self):
            return (self.u, self.v)
        def __set__(self, uv):
            self.u, self.v = uv

    property rgb:
        def __get__(self):
            return (self.red, self.green, self.blue)
        def __set__(self, rgb):
            self.red, self.green, self.blue = rgb

    property rgba:
        def __get__(self):
            return (self.red, self.green, self.blue, self.alpha)
        def __set__(self, rgba):
            self.red, self.green, self.blue, self.alpha = rgba

    property x:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].x
        def __set__(self, float x):
            self.check_index()
            self._parent._vertex_info[self._index].x = x

    property y:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].y
        def __set__(self, float y):
            self.check_index()
            self._parent._vertex_info[self._index].y = y

    property u:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].u
        def __set__(self, float u):
            self.check_index()
            self._parent._vertex_info[self._index].u = u

    property v:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].v
        def __set__(self, float v):
            self.check_index()
            self._parent._vertex_info[self._index].v = v

    property red:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].red/255.0
        def __set__(self, float red):
            self.check_index()
            self._parent._vertex_info[self._index].red = _to255(red)

    property green:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].green/255.0
        def __set__(self, float green):
            self.check_index()
            self._parent._vertex_info[self._index].green = _to255(green)

    property blue:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].blue/255.0
        def __set__(self, float blue):
            self.check_index()
            self._parent._vertex_info[self._index].blue = _to255(blue)

    property alpha:
        def __get__(self):
            self.check_index()
            return self._parent._vertex_info[self._index].alpha/255.0
        def __set__(self, float alpha):
            self.check_index()
            self._parent._vertex_info[self._index].alpha = _to255(alpha)

    def __getitem__(self, int index):
        name = "x y u v red green blue alpha".split()[index]
        return getattr(self, name)

    def __setitem__(self, int index, float value):
        name = "x y u v red green blue alpha".split()[index]
        setattr(self, name, value)

    def __len__(self):
        return 8

__docs_all__ = ('VertexArray VertexInfo VertexArrayIndexes').split()



import random

import rabbyt
from rabbyt import lerp, wrap
import pygame
import os.path
rabbyt.data_directory = os.path.dirname(__file__)

rabbyt.init_display((640, 480))


sprites = []

r = lambda: random.random()-.5

for i in range(2400):
    s = rabbyt.Sprite("rounded_square.png")
    s.rgba = lerp((.5,.2,1,.2), (0,.8,0,.6), dt=3000*r()+2000, extend="reverse")

    s.x = wrap([-320,320], lerp(r()*640, r()*640, dt=2000, extend="extrapolate"))
    s.y = wrap([-240,240], lerp(r()*480, r()*480, dt=2000, extend="extrapolate"))

    s.scale = lerp(.1, 1, dt=1000*r()+750, extend="reverse")

    s.rot = lerp(0, 360, dt=2000, extend="extrapolate")

    sprites.append(s)

print "Drawing 2400 sprites..."

c = pygame.time.Clock()
last_fps = 0
while not pygame.event.peek(pygame.QUIT):
    c.tick()
    if pygame.time.get_ticks() - last_fps > 1000:
        print "FPS: ", c.get_fps()
        last_fps = pygame.time.get_ticks()
    rabbyt.clear()
    rabbyt.set_time(pygame.time.get_ticks())
    rabbyt.render_unsorted(sprites)
    pygame.display.flip()



from __future__ import division

import random

import rabbyt, rabbyt.collisions
from rabbyt import lerp, wrap

import pygame

rabbyt.init_display((800, 600))


sprites = []

r = lambda: random.random()-.5

for i in range(400):
    s = rabbyt.Sprite(shape=(-2,2,2,-2))

    s.x = wrap([-400,400], lerp(r()*800, r()*800, dt=4000, extend="extrapolate"))
    s.y = wrap([-300,300], lerp(r()*600, r()*600, dt=4000, extend="extrapolate"))

    sprites.append(s)

collision_times = []
c = pygame.time.Clock()
last_fps = 0
while not pygame.event.peek(pygame.QUIT):
    c.tick()
    if pygame.time.get_ticks() - last_fps > 1000:
        print "FPS: ", c.get_fps()
        last_fps = pygame.time.get_ticks()
        average = sum(collision_times)/len(collision_times)
        print "Average ms to find collisions:", average
        collision_times = []
    rabbyt.clear()
    rabbyt.set_time(pygame.time.get_ticks())

    start = pygame.time.get_ticks()
    # With so few sprites so close together, just using brute_force ends up
    # being faster.  If you have more sprites spread further apart, collide
    # should work better.
    collisions = rabbyt.collisions.brute_force(sprites)
    #collisions = rabbyt.collisions.collide(sprites)
    collision_times.append(pygame.time.get_ticks()-start)

    for s1, s2 in collisions:
        s1.rgb = lerp((1,0,0),(1,1,1), dt=400)
        s2.rgb = lerp((1,0,0),(1,1,1), dt=400)

    rabbyt.render_unsorted(sprites)
    pygame.display.flip()



"""
Rabbyt is a fast 2d sprite library for Python.
"""

__credits__ = (
"""
Copyright (C) 2007  Matthew Marshall

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
""")

__author__ = "Matthew Marshall <matthew@matthewmarshall.org>"
__version__ = "0.6"

import heapq
import os.path

from rabbyt._rabbyt import *
from rabbyt.sprites import *
from rabbyt.anims import *
import rabbyt.physics, rabbyt.collisions

extend_types = dict(constant=1, extrapolate=2, repeat=3, reverse=4)

def _handle_time_args(startt, endt, dt):
    if startt is None:
        startt = get_time()
    if endt is None:
        if dt is None:
            raise ValueError("Either dt or endt must be given.")
        endt = startt + dt

    assert startt < endt

    return int(startt), int(endt)


def lerp(start, end, startt=None, endt=None, dt=None, extend="constant"):
    """
    ``lerp(start, end, [startt,] [endt,] [dt,] [extend])``

    Linearly interpolates between ``start`` and ``end`` as time moves from
    ``startt`` to ``endt``.

    ``startt`` is the time to start.  It defaults to ``rabbyt.get_time()``.

    To specify the ending time, use either ``endt``, which is the absolute
    time, or ``dt``, which is relative from ``startt``.

    For example, all of the following are equivalent:

        .. sourcecode:: python

            lerp(0, 1, rabbyt.get_time(), rabbyt.get_time()+1000)
            lerp(0, 1, rabbyt.get_time(), dt=1000)
            lerp(0, 1, dt=1000)

    ``extend`` is a string defining what to do before ``startt`` and after
    ``endt``. Possible values are "constant", "extrapolate", "repeat",
    "reverse".  The default is "constant".  Check out the extend_modes.py
    example for more info.

    ``start`` and ``end`` can either be numbers, or tuples of numbers.  If
    they are tuples, a tuple of anims will be returned.  For example, this
    line:

        .. sourcecode:: python

            sprite.rgba = lerp((0,1,0,.5), (1,0,1,1), dt=1000)

    is equivalent to this:

        .. sourcecode:: python

            sprite.red   = lerp(0, 1, dt=1000)
            sprite.green = lerp(1, 0, dt=1000)
            sprite.blue  = lerp(0, 1, dt=1000)
            sprite.alpha = lerp(.5,1, dt=1000)

    """
    extend = extend_types[extend]

    startt, endt = _handle_time_args(startt, endt, dt)

    try:
        iter(start), iter(end)
    except TypeError:
        return AnimLerp(start, end, startt, endt, extend)
    else:
        return [AnimLerp(s, e, startt, endt, extend)
                for s,e in zip(start, end)]

def wrap(bounds, parent, static=True):
    """
    ``wrap(bounds, parent, static=True) -> AnimWrap or tuple of AnimWraps``

    Wraps a parent ``Anim`` to fit within ``bounds``.  ``bounds`` should be an
    object that supports item access for at least ``bounds[0]`` and
    ``bounds[1]``.  (A list or tuple with a length of 2 would work great.)

    If ``static`` is ``True``, ``bounds`` is only read once and stored in C
    variables for fast access. This is much faster, but doesn't work if
    ``bounds`` is an object you wish to mutate.

    If ``parent`` is a iterable, a tuple of anims will be returned instead
    of a single one.  (This is similar to ``lerp()``.)
    """
    try:
        iter(parent)
    except TypeError:
        return AnimWrap(bounds, parent, static)
    else:
        return tuple([AnimWrap(bounds, p, static) for p in parent])

def bezier3(p0, p1, p2, p3, startt=None, endt=None, dt=None, extend="constant"):
    """
    ``bezier3(p0, p1, p2, p3, [startt,] [endt,] [dt,] [extend])``

    Interpolates along a cubic bezier curve as defined by ``p0``, ``p1``,
    ``p2``, and ``p3``.

    ``startt``, ``endt``, ``dt``, and ``extend`` work as in ``lerp()``.

    ``p0``, ``p1``, ``p2``, and ``p3`` can be tuples, but they must all be the
    same length.
    """
    extend = extend_types[extend]
    startt, endt = _handle_time_args(startt, endt, dt)

    try:
        [iter(p) for p in [p0,p1,p2,p3]]
    except TypeError:
        return AnimStaticCubicBezier(p0, p1, p2, p3, startt, endt, extend)
    else:
        return [AnimStaticCubicBezier(p0, p1, p2, p3, startt, endt, extend)
                for p0, p1, p2, p3 in zip(p0, p1, p2, p3)]


class Scheduler(object):
    """
    ``Scheduler()``

    Scheduler provides... (wait for it...)  scheduling!

    You may create your own scheduler instances, or use the default
    ``rabbyt.scheduler``
    """
    def __init__(self):
        self.heap = []

    def add(self, time, callback):
        """
        ``add(time, callback)``

        Schedules a ``callback`` to be called at a given ``time``.
        """
        heapq.heappush(self.heap, (time, callback))

    def pump(self, time=None):
        """
        ``pump([time])``

        Calls all callbacks that have been scheduled for before ``time``.

        If ``time`` is not given, the value returned by ``rabbyt.get_time()``
        will be used.
        """
        if time is None:
            time = get_time()
        try:
            while self.heap[0][0] <= time:
                heapq.heappop(self.heap)[1]()
        except IndexError, e:
            # If the IndexError was raised due to something other than an
            # empty heap we don't want to silence it.
            if len(self.heap) != 0:
                raise e

scheduler = Scheduler()


def init_display(size=(640, 480)):
    """
    init_display(size=(640, 480))

    This is a small shortcut to create a pygame window, set the viewport, and
    set the opengl attributes need for rabbyt.

    This function depends on pygame.
    """
    import pygame
    pygame.init()
    pygame.display.set_mode(size, pygame.OPENGL | pygame.DOUBLEBUF)
    set_viewport(size)
    set_default_attribs()

_texture_cache = {}

data_directory = ""

def pygame_load_texture(filename, filter=True, mipmap=True):
    """
    ``pygame_load_texture(filename, filter=True, mipmap=True) -> texture_id,
    size``

    Reads an image from a file and loads it as a texture.  Pygame is used for
    reading the image.

    If ``filename`` is a relative path, the working directory is searched
    first, then ``rabbyt.data_directory`` is searched for the file.
    """
    if filename not in _texture_cache:
        import pygame
        if os.path.exists(filename):
            img = pygame.image.load(filename)
        else:
            img = pygame.image.load(os.path.join(data_directory, filename))
        data, size = pygame.image.tostring(img, 'RGBA', True), img.get_size()
        _texture_cache[filename] = load_texture(data, size, "RGBA",
                filter, mipmap), size
    return _texture_cache[filename]
set_load_texture_file_hook(pygame_load_texture)

__all__ = __docs_all__ = ('anims collisions fonts sprites physics vertexarrays '
'Scheduler '
'lerp wrap bezier3 '
'init_display set_viewport set_default_attribs set_gl_color clear '
'get_gl_vendor '
'render_unsorted '
'load_texture update_texture unload_texture pygame_load_texture '
'set_load_texture_file_hook ').split()


from __future__ import division

import ctypes as ct
import pymunk._chipmunk as cp
from pymunk.vec2d import vec2d

import pygame
import rabbyt

from math import cos, sin, radians, degrees
import random
import os.path

rabbyt.data_directory = os.path.dirname(__file__)

COLLTYPE_DEFAULT = 0

def create_cube(space, mass = 5.0, xy = (0,0)):
    p_num = 4
    P_ARR = vec2d * p_num
    p_arr = P_ARR(vec2d(0,0))
    
    sprite = rabbyt.Sprite(xy=xy)
    for i, p in enumerate(sprite.shape):
        p_arr[3-i].x, p_arr[3-i].y = p

    moment = 1000
    body = cp.cpBodyNew(mass, moment)
    body.contents.p = vec2d(*xy)
    cp.cpSpaceAddBody(space, body)
    
    shape = cp.cpPolyShapeNew(body, p_num, p_arr, vec2d(0,0))
    shape.contents.u = 0.5
    shape.contents.collision_type = COLLTYPE_DEFAULT
    cp.cpSpaceAddShape(space, shape)

    anims = rabbyt.physics.chipmonk_body_anims(ct.addressof(body[0]))
    sprite.x, sprite.y, sprite.rot = anims

    return shape, sprite

if __name__ == "__main__":
    rabbyt.init_display((800,600))

    ### Physics stuff
    cp.cpInitChipmunk()
    space = cp.cpSpaceNew()
    space.contents.gravity = vec2d(0.0, -900.0)
    
    cp.cpSpaceResizeStaticHash(space, 50.0, 2000)
    cp.cpSpaceResizeActiveHash(space, 50.0, 100)

    wallBody = cp.cpBodyNew(1e100, 1e100)
    wallShape = cp.cpSegmentShapeNew(wallBody, vec2d(-300, -300),
            vec2d(300, -300), 0.0)
    wallShape.contents.u = 1.0
    wallShape.contents.collision_type = COLLTYPE_DEFAULT
    cp.cpSpaceAddStaticShape(space, wallShape)


    shapes = []
    sprites = []

    for x in range(-200, 200, 30):
        for y in range(-200, 200, 30):
            shape, sprite = create_cube(space, xy = (x+random.random()*10,y))
            shapes.append(shape)
            sprites.append(sprite)


    clock = pygame.time.Clock()
    running = True
    while running:
        clock.tick(40)

        for event in pygame.event.get():
            if event.type ==  pygame.QUIT:
                running = False

        rabbyt.set_time(pygame.time.get_ticks())

        rabbyt.scheduler.pump()

        dt = 1/60
        # This is the only call in the loop with ctypes overhead :)
        cp.cpSpaceStep(space, dt)

        rabbyt.clear()

        rabbyt.render_unsorted(sprites)

        pygame.display.flip()




from __future__ import division

import pygame
import rabbyt

from math import cos, sin, radians
import random
import os.path

rabbyt.data_directory = os.path.dirname(__file__)


class Car(rabbyt.Sprite):
    boost_particles = set()
    dust_particles = set()
    def __init__(self, name):
        rabbyt.Sprite.__init__(self, name+'.png', (-30, -20, 50, 20))

        self.shadow = rabbyt.Sprite(name+"shadow.png", self.shape)
        self.shadow.alpha = .5

        # These three lines make use of a rather experimental feature of rabbyt.
        # At the end of update() I have left commented out a more traditional
        # method of doing the same thing.
        self.shadow.rot = lambda: self.rot
        self.shadow.x = lambda: self.x - 4
        self.shadow.y = lambda: self.y - 5
        # Here is another method with identical results (only faster):
        #self.shadow.rot = self.attrgetter('rot')
        #self.shadow.x = self.attrgetter('x') - 4
        #self.shadow.y = self.attrgetter('y') - 5

        self.dust_r = (-15, 10)
        self.dust_l = (-15,-10)

        self.accelerating = False
        self.turning_right = False
        self.turning_left = False

        self.boost_endtime = 0
        self.boost_rot = 0

        self.boost_length = 1000

        self.xy = [0,0]
        self.velocity = [0,0]
        self.rot = 0

    def boost(self):
        if self.boost_endtime > rabbyt.get_time():
            return
        self.boost_rot = self.rot
        self.boost_endtime = rabbyt.get_time() + self.boost_length

    def update(self):
        if self.turning_right:
            self.rot -= 5
        if self.turning_left:
            self.rot += 5

        a = [0.0,0.0]
        if self.boost_endtime > rabbyt.get_time():
            f = 3*(self.boost_endtime - rabbyt.get_time())/self.boost_length
            a[0] += cos(radians(self.boost_rot))*f
            a[1] += sin(radians(self.boost_rot))*f
            self.create_boost_particle()

        if self.accelerating:
            a[0] += cos(radians(self.rot))*.9
            a[1] += sin(radians(self.rot))*.9
            self.create_dust_particle(self.dust_r)
            self.create_dust_particle(self.dust_l)

        ff = .9 # Friction Factor

        self.velocity[0] *= ff
        self.velocity[1] *= ff

        self.velocity[0] += a[0]
        self.velocity[1] += a[1]

        self.x += self.velocity[0]
        self.y += self.velocity[1]

        #self.shadow.x = self.x - 4
        #self.shadow.y = self.y - 5
        #self.shadow.rot = self.rot


    def create_boost_particle(self):
        s = rabbyt.Sprite(self.texture_id, self.shape)

        lifetime = 500

        s.xy = self.xy
        s.rot = self.rot
        s.scale = rabbyt.lerp(1, 2, dt=lifetime)
        s.alpha = rabbyt.lerp(.8, 0, dt=lifetime)

        Car.boost_particles.add(s)
        rabbyt.scheduler.add(rabbyt.get_time()+lifetime,
                lambda:Car.boost_particles.remove(s))

	lt = 800
	star = rabbyt.Sprite("star2.png")
	x = random.random()*80-40
	y = random.random()*80-40
	star.x = rabbyt.lerp(self.x+x, self.convert_offset((-20,0))[0]+x, dt=lt)
	star.y = rabbyt.lerp(self.y+y, self.convert_offset((-20,0))[1]+y, dt=lt)
       	star.rot = rabbyt.lerp(0, 190*random.choice([-2,-1,1,2]), dt=500, extend="extrapolate")
	star.scale = rabbyt.lerp(random.random()+.2,0, rabbyt.get_time()+lt/2, dt=lt/2)
	star.rgb = 0, .5, .9
	Car.boost_particles.add(star)
	rabbyt.scheduler.add(rabbyt.get_time()+lt,
		lambda:Car.boost_particles.remove(star))


    def create_dust_particle(self, offset):
        s = rabbyt.Sprite("star.png")

        lifetime = 4000

        x, y = self.convert_offset(offset)

        r = random.random  # (shortcut)
        s.x = rabbyt.lerp(x+r()*10-5, x+r()*60-30, dt=lifetime)
        s.y = rabbyt.lerp(y+r()*10-5, y+r()*60-30, dt=lifetime)

        s.rot = rabbyt.lerp(0, 90*random.choice(range(-2,3)), dt=6000)

        s.scale = rabbyt.lerp(1, 4, dt=lifetime)

        now = rabbyt.get_time()
        s.rgba = .7, .5, 0, rabbyt.lerp(.1, 0, now+lifetime/2, now+lifetime)

        Car.dust_particles.add(s)
        rabbyt.scheduler.add(rabbyt.get_time()+lifetime,
                lambda:Car.dust_particles.remove(s))

    def render(self):
        self.shadow.render()
        rabbyt.Sprite.render(self)



if __name__ == "__main__":
    rabbyt.init_display((800,600))

    print """
    This is a simple example for using rabbyt.

    Use the arrow keys to steer.  Press SPACE to boost.
    """

    car1 = Car("car")

    cars = [car1]

    clock = pygame.time.Clock()
    while True:
        clock.tick(40)

        for event in pygame.event.get():
            if event.type ==  pygame.QUIT:
                import sys; sys.exit(0)
            elif event.type == pygame.KEYDOWN:
                if event.key in (pygame.K_ESCAPE, pygame.K_q):
                    import sys
                    sys.exit(0)
                elif event.key == pygame.K_SPACE:
                    car1.boost()

        pressed = pygame.key.get_pressed()
        car1.accelerating = pressed[pygame.K_UP]
        car1.turning_right = pressed[pygame.K_RIGHT]
        car1.turning_left = pressed[pygame.K_LEFT]

        rabbyt.set_time(pygame.time.get_ticks())

        for c in cars:
            c.update()

        rabbyt.scheduler.pump()


        rabbyt.clear((.56, .3, 0, 1))

        for c in cars:
            c.render()

        rabbyt.render_unsorted(Car.dust_particles)

        rabbyt.render_unsorted(Car.boost_particles)

        pygame.display.flip()




import rabbyt
from rabbyt.fonts import Font, FontSprite
import pygame

rabbyt.init_display()

font = Font(pygame.font.Font(pygame.font.get_default_font(), 20))
sprite = FontSprite(font, "hello world")
sprite.rot = rabbyt.lerp(0, 360, dt=6000, extend="repeat")
sprite.x = 100
sprite.rgb = 0,1,1

while not pygame.event.peek(pygame.QUIT):
    rabbyt.clear((0,.5,.5,1))
    rabbyt.set_time(pygame.time.get_ticks())
    sprite.render()
    pygame.display.flip()


import pygame
import rabbyt
import rabbyt.physics

rabbyt.init_display((640,480))

class Cannon(rabbyt.Sprite):
    def __init__(self, xy):
        rabbyt.Sprite.__init__(self, shape=(-10,-10,10,30))
        self.xy = xy
        self.rot = 0

    def fire(self):
        CannonBall(self)
        self.scale = rabbyt.lerp((1,1), (1.4, .6), dt=100, extend="reverse")
        def reset():
            self.scale = 1
        rabbyt.scheduler.add(rabbyt.get_time()+200, reset)

cannon = Cannon((0,0))
sprites = [cannon]


class CannonBall(rabbyt.Sprite):
    def __init__(self, cannon):
        rabbyt.Sprite.__init__(self)

        self.xy = rabbyt.physics.Basic2(cannon.xy,
                cannon.convert_offset((0,200)), (0, -100))

        rabbyt.physics.default_update_manager.append(self)

        sprites.append(self)

        rabbyt.scheduler.add(rabbyt.get_time()+10000,
                lambda: sprites.remove(self))

    def update(self, dt):
        pass


clock = pygame.time.Clock()
running=True
while running:
    dt = clock.tick(40)

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running=False
        elif event.type == pygame.KEYDOWN:
            if event.key in (pygame.K_ESCAPE, pygame.K_q):
                running=False
            elif event.key == pygame.K_SPACE:
                cannon.fire()

    pressed = pygame.key.get_pressed()
    if pressed[pygame.K_RIGHT]:
        cannon.rot -= 5
    if pressed[pygame.K_LEFT]:
        cannon.rot += 5

    rabbyt.set_time(pygame.time.get_ticks())
    rabbyt.scheduler.pump()

    rabbyt.clear()
    rabbyt.physics.update(dt/1000.)

    rabbyt.render_unsorted(sprites)

    pygame.display.flip()





import rabbyt
from rabbyt.fonts import Font, FontSprite
import pygame

rabbyt.init_display()

hello_earthling = (u'\u043f\u0440\u0438\u0432\u0435\u0442 '
            u'\u0437\u0430\u0437\u0435\u043c\u043b\u044f\u0442\u044c')

alphabet = set(hello_earthling)

font = Font(pygame.font.Font(pygame.font.get_default_font(), 20),
                alphabet=alphabet)
sprite = FontSprite(font, hello_earthling)
sprite.rot = 20
sprite.x = 100
sprite.rgb = 0,1,1

while not pygame.event.peek(pygame.QUIT):
    rabbyt.clear((0,.5,.5,1))
    rabbyt.set_time(pygame.time.get_ticks())
    sprite.render()
    pygame.display.flip()


import rabbyt
import pygame

import os.path
rabbyt.data_directory = os.path.dirname(__file__)


rabbyt.init_display((640, 480))

car = rabbyt.Sprite("car.png")

# Sprites have various properties you can change.
car.x = 100
car.rot = 45
car.scale = .5
car.rgb = (1,0,0)

# The full list of properties is: x, y, u, v, rot, scale, red, green, blue, and
# alpha
# There all also a few shortcut properties to allow you to set multiple values
# at a time: xy, uv, rgb, and rgba

while not pygame.event.peek(pygame.QUIT):
    rabbyt.clear((1,1,1))

    car.render()

    pygame.display.flip()



"""
"""

__credits__ = (
"""
Copyright (C) 2007  Matthew Marshall

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
""")

__author__ = "Matthew Marshall <matthew@matthewmarshall.org>"


cdef extern from "math.h":
    cdef float fmodf(float x, float y)
    cdef float cosf(float x)
    cdef float sinf(float x)
    cdef float sqrtf(float x)

cdef int time

def set_time(int t):
    """
    ``set_time(t)``

    Sets the time that rabbyt.get_time() should return.

    If you are using any time based animations, (such as ``rabbyt.lerp`` or
    ``rabbyt.bezier3``,) you should call this function every frame.

    For example, if you are using pygame you can do this::

        rabbyt.set_time(pygame.time.get_ticks())

    Using this function should make it easier to implement a pause feature.
    """
    global time
    time = t

def get_time():
    """
    ``get_time()``

    Gets the time that was last set by ``rabbyt.set_time()``
    """
    global time
    return time

cdef int _get_time():
    return time


cdef class Anim:
    """
    ``Anim()``

    This is the base class for anims.  It shouldn't be instanced directly.

    Performing arithmatic opperations on an anim will result in a new anim that
    will allways be up to date.
    """
    cdef float g(self):
        return 0.0

    def get(self):
        """
        get() -> float

        Gets the value.
        """
        return self.g()

    def __add__(self, other):
        if not isinstance(self, Anim):
            self, other = other, self
        if isinstance(other, (int, long, float)):
            return AnimAddStatic(self, other)
        else:
            return AnimAdd(self, other)

    def __sub__(self, other):
        if isinstance(self, Anim) and isinstance(other, (int, long, float)):
            return AnimAddStatic(self, -other)
        else:
            return AnimSub(self, other)

    def __mul__(self, other):
        return AnimMul(self, other)

    def __div__(self, other):
        return AnimDiv(self, other)


cdef class AnimConst(Anim):
    """
    ``AnimConst(value)``

    An anim that isn't animated.

    This is mostly here so that constant values can be used with the same
    interface as Anim.
    """
    cdef float v

    def __init__(self, float v):
        self.v = v

    cdef float g(self):
        return self.v

cdef float extend_t(float t, int mode):
    if mode == 1: # constant
        if t < 0:
            t = 0
        elif t > 1:
            t = 1
    elif mode == 2: # extrapolate
        pass
    elif mode == 3: # repeat
        if t >= 1:
            t = t - (<int>t)
        elif t < 0:
            t = 1 + t - (<int>t)
    elif mode == 4: # reverse
        if t < 0:
            t = -t
        if (<int>t) % 2 == 1:
            t = 1 - (t - (<int>t))
        else:
            t = t - (<int>t)
    return t

cdef class AnimLerp(Anim):
    cdef float start
    cdef float end
    cdef int startt
    cdef int endt
    cdef int extend
    cdef float one_over_dt

    def __init__(self, float start, float end, int startt, int endt,
            int extend):
        self.start = start
        self.end = end
        self.startt = startt
        self.endt = endt
        self.extend = extend
        self.one_over_dt = 1/<float>(self.endt-self.startt)

    cdef float g(self):
        cdef float t
        t = extend_t((_get_time()-self.startt)*self.one_over_dt, self.extend)

        return (self.end-self.start) * t + self.start

cdef class AnimStaticCubicBezier(Anim):
    cdef float p0
    cdef int startt, endt
    cdef int extend
    cdef float one_over_dt
    cdef float a, b, c

    def __init__(self, float p0, float p1, float p2, float p3, int startt,
            int endt, int extend):
        self.p0 = p0
        self.startt = startt
        self.endt = endt
        self.extend = extend
        self.one_over_dt = 1/<float>(endt-startt)
        self.c = 3.0 * (p1 - p0)
        self.b = 3.0 * (p2 - p1) - self.c
        self.a = p3 - p0 - self.c - self.b

    cdef float g(self):
        cdef float t, t2, t3
        t = extend_t((_get_time()-self.startt)*self.one_over_dt, self.extend)
        t2 = t * t
        t3 = t2 * t
        return self.a*t3 + self.b*t2 + self.c*t + self.p0

cdef class AnimWrap(Anim):
    """
    ``AnimWrap(bounds, parent, static=True)``

    An anim that returns another anim wrapped within two bounds.

    You might want to use ``rabbyt.wrap()`` instead.

   ``bounds`` is the bounds that the value should be wrapped within.  It can
    be anything supporting item access with a length of at least two.

    ``parent`` is the ``Anim`` that is being wrapped.

    If static is ``True``, ``bounds[0]`` and ``bounds[1]`` are read only once
    and stored as variables in c.  This is much faster, but doesn't work if
    ``bounds`` is an object you wish to mutate.
    """
    cdef Anim parent
    cdef object bounds
    cdef unsigned char is_static
    cdef float s_bounds[2]

    def __init__(self, bounds, parent, static=True):
        if static:
            self.is_static = 1
            self.s_bounds[0] = bounds[0]
            self.s_bounds[1] = bounds[1]
            self.bounds = None
        else:
            self.is_static = 0
            self.bounds = bounds

        self.parent = to_Anim(parent)

    cdef float g(self):
        cdef float p, b1, b2, d
        if self.is_static == 1:
            b1 = self.s_bounds[0]
            b2 = self.s_bounds[1]
        else:
            b1 = self.bounds[0]
            b2 = self.bounds[1]

        p = self.parent.g()

        d = b2 - b1

        p = fmodf(p-fmodf(b1,d), d)
        if p < 0:
            p = p + d

        return p + b1

cdef class AnimAddStatic(Anim):
    cdef float s
    cdef Anim a

    def __init__(self, Anim a not None, float s):
        self.a = a
        self.s = s

    cdef float g(self):
        return self.a.g() + self.s


cdef class _AnimArithmatic(Anim):
    cdef Anim a1, a2

    def __init__(self, a1, a2):
        self.a1 = to_Anim(a1)
        self.a2 = to_Anim(a2)

cdef class AnimAdd(_AnimArithmatic):
    cdef float g(self):
        return self.a1.g() + self.a2.g()

cdef class AnimSub(_AnimArithmatic):
    cdef float g(self):
        return self.a1.g() - self.a2.g()

cdef class AnimMul(_AnimArithmatic):
    cdef float g(self):
        return self.a1.g() * self.a2.g()

cdef class AnimDiv(_AnimArithmatic):
    cdef float g(self):
        return self.a1.g() / self.a2.g()

cdef class AnimPyFunc(Anim):
    """
    ``AnimPyFunc(function, cache=False)``

    An anim that calls a python function, using the returned value.

    function is the callback called to retrieve the value.  It should
    return a float.

    If ``cache`` is ``True``, the result returned by function will be
    cached for as long as the time (as set by ``rabbyt.set_time()``) doesn't
    change. This could provide good speedup if the value is read multiple
    times per frame.
    """
    cdef object function
    cdef int cache_output # would be bool if pyrex allowed it.
    cdef float cache
    cdef int cache_time
    def __init__(self, function, cache=False):
        self.function = function
        self.cache_output = cache

    cdef float g(self):
        if self.cache_output:
            if _get_time() != self.cache_time:
                self.cache = self.function()
                self.cache_time = _get_time()
            return self.cache
        else:
            return self.function()

cdef class AnimProxy(Anim):
    """
    ``AnimProxy(value, cache=False)``

    An anim that simply returns the value of another anim.
    """
    cdef int cache_output
    cdef float cache
    cdef int cache_time
    cdef Anim _value
    def __init__(self, value, cache=False):
        self.cache_output = cache
        self.value = value

    property value:
        def __get__(self):
            return self._value.g()
        def __set__(self, v):
            self._value = to_Anim(v)
            self.cache_time = 0

    cdef float g(self):
        if not self.cache_output or _get_time() != self.cache_time:
            self.cache = self._value.g()
            self.cache_time = _get_time()
        return self.cache

def to_Anim(v):
    """
    ``to_Anim(value) -> Anim subclass instance``

    This is a convenience function to get ``Anim`` for a value.

    If ``value`` is already an ``Anim``, it is returned directly.

    If ``value`` is callable, it is wrapped in an ``AnimPyFunc``.

    Otherwise, ``value`` is wrapped in an ``AnimConst``.
    """
    cdef Anim dv
    if isinstance(v, Anim):
        dv = v
    elif callable(v):
        dv = AnimPyFunc(v)
    else:
        dv = AnimConst(v)
    return dv

__docs_all__ = ('set_time get_time Anim AnimConst AnimPyFunc AnimProxy '
'to_Anim').split()



import rabbyt
import rabbyt.physics
import pygame

import os.path
rabbyt.data_directory = os.path.dirname(__file__)

rabbyt.init_display((640, 480))

car = rabbyt.Sprite("car.png")
car.xy = rabbyt.physics.Basic2(xy=(-100, 0), vxy=(100,200), axy=(0,-100))

c = pygame.time.Clock()
while not pygame.event.peek(pygame.QUIT):
    dt = c.tick(100)
    rabbyt.set_time(pygame.time.get_ticks())

    rabbyt.physics.update(dt/1000.0)

    rabbyt.clear((1,1,1))

    car.render()

    pygame.display.flip()



"""
The most interesting part of this module is probably the ``Sprite`` class.

If you need more specialized rendering, try subclassing from ``BaseSprite``.
"""

__credits__ = (
"""
Copyright (C) 2007  Matthew Marshall

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
""")

__author__ = "Matthew Marshall <matthew@matthewmarshall.org>"

cdef extern from "math.h":
    cdef float fmodf(float x, float y)
    cdef float cosf(float x)
    cdef float sinf(float x)
    cdef float sqrtf(float x)

cdef extern from "GL/gl.h":
    ctypedef float GLfloat
    ctypedef float GLclampf
    ctypedef unsigned int GLenum
    ctypedef unsigned int GLbitfield
    ctypedef int GLint
    ctypedef unsigned int GLuint
    ctypedef int GLsizei
    ctypedef double GLdouble
    ctypedef double GLclampd
    ctypedef void GLvoid

    cdef int GL_SMOOTH
    cdef int GL_COLOR_BUFFER_BIT
    cdef int GL_BLEND
    cdef int GL_SRC_ALPHA
    cdef int GL_ONE_MINUS_SRC_ALPHA
    cdef int GL_TEXTURE_2D
    cdef int GL_QUADS
    cdef int GL_MODELVIEW
    cdef int GL_RGBA
    cdef int GL_RGB
    cdef int GL_NEAREST
    cdef int GL_LINEAR
    cdef int GL_TEXTURE_MAG_FILTER
    cdef int GL_TEXTURE_MIN_FILTER
    cdef int GL_TEXTURE_ENV
    cdef int GL_TEXTURE_ENV_MODE
    cdef int GL_MODULATE
    cdef int GL_LINEAR_MIPMAP_NEAREST
    cdef int GL_UNSIGNED_BYTE
    cdef int GL_PROJECTION
    cdef int GL_FLAT
    cdef int GL_FLOAT
    cdef int GL_POLYGON_SMOOTH

    cdef void glTranslatef(GLfloat x, GLfloat y, GLfloat z)
    cdef void glRotatef(GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
    cdef void glScalef(GLfloat x, GLfloat y, GLfloat z)
    cdef void glEnable(GLenum cap)
    cdef void glDisable(GLenum cap)
    cdef void glClear(GLbitfield mask)
    cdef void glShadeModel(GLenum mode)
    cdef void glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
    cdef void glBlendFunc(GLenum sfactor, GLenum dfactor)
    cdef void glBindTexture(GLenum target, GLuint texture)
    cdef void glColor4f(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha)
    cdef void glBegin(GLenum mode)
    cdef void glTexCoord2f(GLfloat s, GLfloat t)
    cdef void glVertex2f(GLfloat x, GLfloat y)
    cdef void glEnd()
    cdef void glViewport(GLint x, GLint y, GLsizei width, GLsizei height)
    cdef void glOrtho(GLint bottom, GLint left, GLint top, GLint bottom, GLint front, GLint back)
    cdef void glMatrixMode(GLenum mode)
    cdef void glLoadIdentity()
    cdef void glTexParameteri(GLenum target, GLenum pname, GLint param)
    cdef void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, GLvoid *pixels)
    cdef void glGenTextures(GLsizei n, GLuint *textures)
    cdef void glDeleteTextures(GLsizei n, GLuint *textures)
    cdef void glTexEnvf(GLenum target, GLenum pname, GLfloat param)
    cdef void glPushMatrix()
    cdef void glPopMatrix()


from anims cimport Anim, AnimConst

from rabbyt.anims import to_Anim

cdef float pi, pi_over_180
pi = 3.1415926535897931
pi_over_180 = pi / 180.0

cdef struct Point2d:
    float x,y

cdef struct float2:
    float a, b

cdef class BaseSprite:
    """
    ``BaseSprite(...)``

    This class provides some basic functionality for sprites:

    * transformations (x, y, rot, scale)
    * color (red, green, blue, alpha)
    * bounding_radius (for collision detection)

    ``BaseSprite`` doesn't render anything itself  You'll want to subclass it
    and override either ``render()`` or ``render_after_transform()``.

    You can pass any of the ``BaseSprite`` properties as keyword arguments.
    (``x``, ``y``, ``xy``, etc.)
    """
    cdef object __weakref__

    cdef Anim _r, _g, _b, _a
    cdef Anim _x, _y
    cdef Anim _rot

    # If _scale_y is None, _scale will be used for both x and y scaling.
    cdef Anim _scale, _scale_y

    cdef float _bounding_radius

    def __new__(self, *args, **kwargs):
        # We assign these in __new__ to avoid segfaults from the properties
        # being read before assigned for the first time.
        self._r = AnimConst(1)
        self._g = AnimConst(1)
        self._b = AnimConst(1)
        self._a = AnimConst(1)

        self._x = AnimConst(0)
        self._y = AnimConst(0)

        self._rot = AnimConst(0)
        self._scale = AnimConst(1)
        self._scale_y = None

    def __init__(self, **kwargs):
        names = "xy x y rgba red green blue alpha scale rot".split()
        for name, value in kwargs.items():
            if name not in names:
                raise TypeError("unexpected keyword argument %r" % name)
            setattr(self, name, value)

    property xy:
        """
        A tuple of the x and y coordinates
        """
        def __get__(self):
            return (self._x.g(), self._y.g())
        def __set__(self, p):
            self._x = to_Anim(p[0])
            self._y = to_Anim(p[1])

    property x:
        """ x coordinate """
        def __get__(self):
            return self._x.g()
        def __set__(self, x):
            self._x = to_Anim(x)

    property y:
        """ y coordinate """
        def __get__(self):
            return self._y.g()
        def __set__(self, y):
            self._y = to_Anim(y)

    property rgb:
        """ red, green, and blue color components """
        def __get__(self):
            return (self.red, self.green, self.blue)
        def __set__(self, c):
            self.red, self.green, self.blue = c

    property rgba:
        """ red, green, blue, and alpha color components """
        def __get__(self):
            return (self.red, self.green, self.blue, self.alpha)
        def __set__(self, c):
            self.red, self.green, self.blue, self.alpha = c

    property red:
        """ red color component """
        def __get__(self):
            return self._r.g()
        def __set__(self, r):
            self._r = to_Anim(r)
    property green:
        """ green color component """
        def __get__(self):
            return self._g.g()
        def __set__(self, g):
            self._g = to_Anim(g)
    property blue:
        """ blue color component """
        def __get__(self):
            return self._b.g()
        def __set__(self, b):
            self._b = to_Anim(b)
    property alpha:
        """ alpha color component """
        def __get__(self):
            return self._a.g()
        def __set__(self, a):
            self._a = to_Anim(a)

    property scale:
        """
        scale

        ``1.0`` is normal size; ``0.5`` is half size, ``2.0`` is double
        size... you get the point.

        You can scale the x and y axises independently by assigning a tuple of
        length two.
        """
        def __get__(self):
            if self._scale_y is not None:
                return (self._scale.g(), self._scale_y.g())
            else:
                return self._scale.g()
        def __set__(self, s):
            try:
                iter(s)
            except:
                self._scale = to_Anim(s)
                self._scale_y = None
            else:
                self._scale = to_Anim(s[0])
                self._scale_y = to_Anim(s[1])

    property rot:
        """ rotation """
        def __get__(self):
            return self._rot.g()
        def __set__(self, r):
            self._rot = to_Anim(r)

    property bounding_radius:
        """
        bounding_radius

        This should be the distance of the farthest point from the center.  It
        can be used for collision detection.
        """
        def __get__(self):
            return self._bounding_radius
        def __set__(self, float r):
            self._bounding_radius = r

    property bounding_radius_squared:
        """
        bounding_radius_squared

        This is just like ``bounding_radius``, only it's squared.  (duh)

        ``bounding_radius`` and ``bounding_radius_squared`` are automatically
        kept in sync with each other.
        """
        def __get__(self):
            return self._bounding_radius * self._bounding_radius
        def __set__(self, float r2):
            self._bounding_radius = sqrtf(r2)


    cdef Point2d _convert_offset(self, float ox, float oy):
        cdef float x, y, sx, sy, r, co, si
        cdef Point2d out
        x = self._x.g()
        y = self._y.g()
        sx = self._scale.g()
        if self._scale_y is None:
            sy = sx
        else:
            sy = self._scale_y.g()
        r = self._rot.g()

        co = cosf(r*pi_over_180)
        si = sinf(r*pi_over_180)
        out.x = (ox*sx*co - oy*sy*si) + x
        out.y = (ox*sx*si + oy*sy*co) + y
        return out

    def convert_offset(self, offset):
        """
        ``convert_offset((x, y)) -> (x, y)``

        Converts coordinates relative to this sprite to global coordinates,
        including rotation and scaling.
        """
        cdef Point2d c_offset, out_offset
        c_offset = self._convert_offset(offset[0], offset[1])
        return (c_offset.x, c_offset.y)

    def attrgetter(self, name):
        """
        ``attrgetter(name)``

        Returns an anim that returns the value of the given attribute name.

        Perhaps this is easiest to see with an example.  The following two lines
        will do the same thing, only the second is much, much faster:

            .. sourcecode:: python

                sprite.x = lambda: other_sprite.x

                sprite.x = other_sprite.attrgetter("x")

        The anim returned by attrgetter is implemented in Pyrex (C) and
        will retrieve the value without doing a python attribute lookup.

        This doesn't work for all attributes -- only ``x``, ``y``, ``scale``,
        ``rot``, ``red``, ``green``, ``blue``, and ``alpha``.  (Subclasses
        might add others.)
        """
        return _base_sprite_attr_getters[name](self)

    def render(self):
        """
        ``render()``

        Renders the sprite.

        By default, this method will transform the OpenGL modelview matrix
        according to ``x``, ``y``, ``scale``, and ``rot``, and call
        ``render_after_transform()``.

        If you want transformations to be handled for you, leave this method and
        override ``render_after_transform()``.  Otherwise, override
        ``render()``.
        """
        cdef float x, y, sx, sy, r

        x = self._x.g()
        y = self._y.g()
        sx = self._scale.g()
        if self._scale_y is None:
            sy = sx
        else:
            sy = self._scale_y.g()
        r = self._rot.g()

        if x != 0 or y != 0 or sx != 1 or sy != 1 or r != 0:
            glPushMatrix()
            try:
                glTranslatef(x, y, 0)
                glRotatef(r,0,0,1)
                glScalef(sx, sy, 1)
                self.render_after_transform()
            finally:
                glPopMatrix()
        else:
            self.render_after_transform()

    def render_after_transform(self):
        """
        ``render_after_transform()``

        This method is called by ``BaseSprite.render()`` after transformations
        have been applied.

        If you don't want to mess with doing transformations yourself, you can
        override this method instead of ``render()``.
        """
        pass



cdef class _AnimBaseSpriteAttr_(Anim):
    cdef BaseSprite sprite
    def __init__(self, BaseSprite sprite not None):
        self.sprite = sprite

cdef class _AnimBaseSpriteAttr_x(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._x.g()

cdef class _AnimBaseSpriteAttr_y(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._y.g()

cdef class _AnimBaseSpriteAttr_rot(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._rot.g()

cdef class _AnimBaseSpriteAttr_scale(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._scale.g()

cdef class _AnimBaseSpriteAttr_red(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._red.g()

cdef class _AnimBaseSpriteAttr_green(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._green.g()

cdef class _AnimBaseSpriteAttr_blue(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._blue.g()

cdef class _AnimBaseSpriteAttr_alpha(_AnimBaseSpriteAttr_):
    cdef float g(self):
        return self.sprite._alpha.g()


_base_sprite_attr_getters = dict(
    x=_AnimBaseSpriteAttr_x,
    y=_AnimBaseSpriteAttr_y,
    rot=_AnimBaseSpriteAttr_rot,
    scale=_AnimBaseSpriteAttr_scale,
    red=_AnimBaseSpriteAttr_red,
    green=_AnimBaseSpriteAttr_green,
    blue=_AnimBaseSpriteAttr_blue,
    alpha=_AnimBaseSpriteAttr_alpha)


cdef class Sprite(BaseSprite):
    """
    ``Sprite(texture=None, shape=None, tex_shape=(0,1,1,0), ...)``

    This class provides a basic, four point, textured sprite.

    All arguments are optional.

    ``texture`` should be either the filename of an image or an OpenGL
    texture id.  If ``shape`` is not given it will default to the
    dimensions of the texture if they are available.  For more information
    on ``shape`` and ``tex_shape`` read the docstrings for ``Sprite.shape``
    and ``Sprite.tex_shape``

    Additionally, you can pass values for most of the properties as keyword
    arguments.  (``x``, ``y``, ``xy``, ``u``, ``v``, ``uv``, etc...)
    """

    cdef Point2d vertexes[4]
    cdef Point2d texcoords[4]

    cdef Anim _u, _v

    cdef int _tex_id

    def __new__(self, *args, **kwargs):
        self._u = AnimConst(0)
        self._v = AnimConst(0)

    def __init__(self, texture=None, shape=None, tex_shape=(0,1,1,0),
            **kwargs):
        from rabbyt._rabbyt import load_texture_file_hook
        if isinstance(texture, basestring):
            self.texture_id, tex_size = load_texture_file_hook(texture)
        elif isinstance(texture, (int, long)):
            self.texture_id = texture
            tex_size = None
        elif texture != None:
            raise ValueError("texture should be either an int or str.")

        if not shape:
            if tex_size:
                w, h = tex_size
                shape = [-w/2, h/2, w/2, -h/2]
            else:
                s = 10.
                shape = [s, s, -s, -s]
        self.shape = shape

        self.tex_shape = tex_shape

        for key in list(kwargs):
            if key in "uv u v left right top bottom".split():
                setattr(self, key, kwargs.pop(key))
        BaseSprite.__init__(self, **kwargs)



    property shape:
        """
        The shape of the sprite.

        This must either be of the form ``[left, top, right, bottom]``, or a
        list of four coordinates, eg. ``[(0,0), (20,0), (20,20), (0,20)]``

        ``[-10, -10, 10, 10]`` is the default.

        When you assign to ``shape``, ``bounding_radius`` is automatically set
        to the distance of the farthest coordinate.
        """
        def __get__(self):
            return ((self.vertexes[0].x, self.vertexes[0].y),
                    (self.vertexes[1].x, self.vertexes[1].y),
                    (self.vertexes[2].x, self.vertexes[2].y),
                    (self.vertexes[3].x, self.vertexes[3].y))
        def __set__(self, s):
            try:
                s[0][0]
            except TypeError:
                l, t, r, b = s
                s = [(l, t), (r, t), (r, b), (l, b)]
            assert len(s) == 4 # Only quads supported now.
            cdef int i
            cdef float d
            for i, v in enumerate(s):
                self.vertexes[i].x = v[0]
                self.vertexes[i].y = v[1]
            self.bounding_radius = 0
            for i from 0 <= i < 4:
                d = self.vertexes[i].x**2 + self.vertexes[i].y**2
                if d > self.bounding_radius_squared:
                    self.bounding_radius_squared = d

    property tex_shape:
        """
        This defines how a texture is mapped onto the sprite.

        Like ``Sprite.shape``, you can give either
        ``[left, top, right, bottom]`` or a list of coordinates.

        The default is ``[0, 1, 1, 0]``, which uses the entire texture.
        """
        def __get__(self):
            return ((self.texcoords[0].x, self.texcoords[0].y),
                    (self.texcoords[1].x, self.texcoords[1].y),
                    (self.texcoords[2].x, self.texcoords[2].y),
                    (self.texcoords[3].x, self.texcoords[3].y))
        def __set__(self, s):
            try:
                s[0][0]
            except TypeError:
                l, t, r, b = s
                s = [(l, t), (r, t), (r, b), (l, b)]
            assert len(s) == 4 # Only quads supported now.
            for i, coord in enumerate(s):
                self.texcoords[i].x = coord[0]
                self.texcoords[i].y = coord[1]


    property u:
        """ u texture offset """
        def __get__(self):
            return self._u.g()
        def __set__(self, u):
            self._u = to_Anim(u)

    property v:
        """ v texture offset """
        def __get__(self):
            return self._v.g()
        def __set__(self, v):
            self._v = to_Anim(v)

    property uv:
        """
        The values of ``u`` and ``v`` are added to ``tex_shape``.  This could
        be useful if you have multiple frames of an animation on a single
        texture.  Set ``tex_shape`` to the shape of a single frame and set
        ``uv``, to select each frame.
        """
        def __get__(self):
            return (self._u.g(), self._v.g())
        def __set__(self, p):
            self._u = to_Anim(p[0])
            self._v = to_Anim(p[1])

    property texture_id:
        """
        ``texture_id`` is a number as returned by ``rabbyt.load_texture()``.
        (But you can use any loaded OpenGL texture.)

        If ``texture_id`` is ``0``, texturing will be disabled for this sprite.
        """
        def __get__(self):
            return self._tex_id
        def __set__(self, int t):
            self._tex_id = t

    def render(self):
        """
        ``render()``

        Renders the sprite onto the screen.

        This method depends on the global OpenGL state.  Keep that in mind if
        things seem to go crazy.
        """
        if self._tex_id != 0:
            glEnable(GL_TEXTURE_2D)
            glBindTexture(GL_TEXTURE_2D, self._tex_id)
        else:
            glDisable(GL_TEXTURE_2D)
        glColor4f(self._r.g(), self._g.g(), self._b.g(), self._a.g())

        cdef float x, y, u, v, sx, sy, r
        x = self._x.g()
        y = self._y.g()
        u = self._u.g()
        v = self._v.g()
        sx = self._scale.g()
        if self._scale_y is None:
            sy = sx
        else:
            sy = self._scale_y.g()
        r = self._rot.g()

        cdef int i
        cdef float vx, vy, co, si

        glBegin(GL_QUADS)
        if r == 0:
            glTexCoord2f(self.texcoords[0].x+u,self.texcoords[0].y+v)
            glVertex2f(self.vertexes[0].x*sx+x,self.vertexes[0].y*sy+y)
            glTexCoord2f(self.texcoords[1].x+u,self.texcoords[1].y+v)
            glVertex2f(self.vertexes[1].x*sx+x,self.vertexes[1].y*sy+y)
            glTexCoord2f(self.texcoords[2].x+u,self.texcoords[2].y+v)
            glVertex2f(self.vertexes[2].x*sx+x,self.vertexes[2].y*sy+y)
            glTexCoord2f(self.texcoords[3].x+u,self.texcoords[3].y+v)
            glVertex2f(self.vertexes[3].x*sx+x,self.vertexes[3].y*sy+y)
        else:
            r = r * pi_over_180
            co = cosf(r)
            si = sinf(r)
            for i from 0 <= i < 4:
                glTexCoord2f(self.texcoords[i].x+u,self.texcoords[i].y+v)
                vx = self.vertexes[i].x*sx
                vy = self.vertexes[i].y*sy
                glVertex2f((vx*co - vy*si)+x, (vx*si + vy*co)+y)
        glEnd()

    def attrgetter(self, name):
        """
        ``attrgetter(name)``

        Returns an anim that returns the value of the given attribute name.

        (See the docs for ``BaseSprite.attrgetter()`` for more information.)

        Sprite adds support for ``u`` and ``v``, in addition to the attributes
        supported by ``BaseSprite``.
        """
        if name == "u":
            return _AnimSpriteAttr_u(self)
        elif name == "v":
            return _AnimSpriteAttr_v(self)
        else:
            return BaseSprite.attrgetter(self, name)


    cdef float2 _bounds_x(self):
            cdef float2 bounds
            cdef float r, co, si, x, s
            cdef int i
            s = self._scale.g()
            r = self._rot.g() * pi_over_180
            co = cosf(r)
            si = sinf(r)

            for i from 0 <= i < 4:
                x = (self.vertexes[i].x*co - self.vertexes[i].y*si)*s
                if i == 0:
                    bounds.a = x
                    bounds.b = x
                else:
                    if bounds.a > x:
                        bounds.a = x
                    if bounds.b < x:
                        bounds.b = x
            return bounds

    cdef float2 _bounds_y(self):
            cdef float2 bounds
            cdef float r, co, si, y, s
            cdef int i
            if self._scale_y is None:
                s = self._scale.g()
            else:
                s = self._scale_y.g()
            r = self._rot.g() * pi_over_180
            co = cosf(r)
            si = sinf(r)

            for i from 0 <= i < 4:
                y = (self.vertexes[i].x*si + self.vertexes[i].y*co)*s
                if i == 0:
                    bounds.a = y
                    bounds.b = y
                else:
                    if bounds.a > y:
                        bounds.a = y
                    if bounds.b < y:
                        bounds.b = y
            return bounds

    property left:
        """ x coordinate of the left side of the sprite """
        def __get__(self):
            return self._bounds_x().a + self._x.g()
        def __set__(self, x):
            self.x = x - self._bounds_x().a

    property right:
        """ x coordinate of the right side of the sprite """
        def __get__(self):
            return self._bounds_x().b + self._x.g()
        def __set__(self, x):
            self.x = x - self._bounds_x().b

    property bottom:
        """ y coordinate of the bottom of the sprite """
        def __get__(self):
            return self._bounds_y().a + self._y.g()
        def __set__(self, y):
            self.y = y - self._bounds_y().a

    property top:
        """ y coordinate of the top of the sprite """
        def __get__(self):
            return self._bounds_y().b + self._y.g()
        def __set__(self, y):
            self.y = y - self._bounds_y().b


cdef class _AnimSpriteAttr_(Anim):
    cdef BaseSprite sprite
    def __init__(self, Sprite sprite not None):
        self.sprite = sprite

cdef class _AnimSpriteAttr_u(_AnimSpriteAttr_):
    cdef float g(self):
        return self.sprite._u.g()

cdef class _AnimSpriteAttr_v(_AnimSpriteAttr_):
    cdef float g(self):
        return self.sprite._v.g()

__docs_all__ = ('BaseSprite Sprite').split()


from math import hypot

import pygame
import rabbyt

rabbyt.init_display((640,480))
rabbyt.set_viewport((640,480), projection=(0,0,640,480))

control_points = [rabbyt.Sprite(xy=xy) for xy in
        [(100,100),(200,50),(300,150),(400,100)]]
grabbed_point = None

path_followers = []
def generate_followers():
    global path_followers
    p = [c.xy for c in control_points]
    path_followers = [rabbyt.Sprite(xy=rabbyt.bezier3(p[0], p[1], p[2], p[3],
            i*200, i*200 + 2000, extend="repeat")) for i in range(10)]

generate_followers()

print "Click and drag the control points."

clock = pygame.time.Clock()
running=True
while running:
    clock.tick(40)

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running=False
        elif event.type == pygame.KEYDOWN:
            if event.key in (pygame.K_ESCAPE, pygame.K_q):
                running=False
        elif event.type == pygame.MOUSEBUTTONDOWN:
            for c in control_points:
                if hypot(c.x-event.pos[0], c.y-event.pos[1]) < 20:
                    grabbed_point = c
                    break
        elif event.type == pygame.MOUSEMOTION:
            if grabbed_point:
                grabbed_point.xy = event.pos
                generate_followers()
        elif event.type == pygame.MOUSEBUTTONUP:
            grabbed_point = None

    rabbyt.set_time(pygame.time.get_ticks())
    rabbyt.clear()
    rabbyt.render_unsorted(control_points)
    rabbyt.render_unsorted(path_followers)
    pygame.display.flip()



import rabbyt
import pygame

import os.path
# We do this so that you don't have to be in the same directory as the script
# to use it.
rabbyt.data_directory = os.path.dirname(__file__)

# Make sure that you allways have a valid OpenGL context before creating a
# sprite.  This init function will do that for you:
rabbyt.init_display((640, 480))

# Creating a sprite is easy.  Just give it an image filename.
car = rabbyt.Sprite("car.png")

while not pygame.event.peek(pygame.QUIT):
    rabbyt.clear((1,1,1))

    car.render()

    pygame.display.flip()



import random

import rabbyt
import pygame

from rabbyt.vertexarrays import VertexArray

rabbyt.init_display((640, 480))

va = VertexArray()

r = random.random

print "Generating 20,000 quads..."
start = pygame.time.get_ticks()
for i in range(20000):
    x = r()*640 - 320
    y = r()*480 - 240
    va.append((x-5, y+5, 0, 0, r(), r(), r(), r()))
    va.append((x+5, y+5, 0, 0, r(), r(), r(), r()))
    va.append((x+5, y-5, 0, 0, r(), r(), r(), r()))
    va.append((x-5, y-5, 0, 0, r(), r(), r(), r()))
print pygame.time.get_ticks() - start

c = pygame.time.Clock()
last_fps = 0
while not pygame.event.peek(pygame.QUIT):
    c.tick()
    if pygame.time.get_ticks() - last_fps > 1000:
        print "FPS: ", c.get_fps()
        last_fps = pygame.time.get_ticks()
    rabbyt.clear()
    va.render()
    pygame.display.flip()






from anims cimport Anim, AnimConst

from rabbyt.anims import to_Anim

import weakref

cdef extern from "math.h":
    cdef float fmodf(float x, float y)
    cdef float cosf(float x)
    cdef float sinf(float x)
    cdef float sqrtf(float x)
    cdef float atan2f(float, float)

cdef float pi, pi_over_180, oneeighty_over_pi
pi = 3.1415926535897931
pi_over_180 = pi / 180.0
oneeighty_over_pi = 180.0 / pi

cdef class Basic1(Anim):
    """
    ``Basic1(p=0, v=0, a=0)``

    Provides basic physics (position, velocity, and acceleration,) in a single
    dimention.

    Basic1 instances can be used as anims, so you can do this for
    example:

        .. sourcecode:: python

            sprite.y = Basic1(10, a=-9.81)

    (The sprite will start at a height of ten, then fall with an acceleration
    of -9.81 world units per second.)

    Uppon creation, a ``Basic1`` instance will automatically add itself to the
    global update manager.  This frees you from having to keep track of
    it yourself.  Just use ``rabbyt.physics.update(dt)`` to update everything.
    """
    cdef object __weakref__

    cdef float _p, _v
    cdef Anim _a

    def __new__(self, *args, **kwargs):
        self._a = AnimConst(0.0)

    def __init__(self, p=0, v=0, a=0):
        """
        """
        self.p = p
        self.v = v
        self.a = a
        default_update_manager.append(self)

    property p:
        """
        position
        """
        def __get__(self):
            return self._p
        def __set__(self, float p):
            self._p = p

    property v:
        """
        velocity
        """
        def __get__(self):
            return self._v
        def __set__(self, v):
            self._v = v

    property a:
        """
        acceleration

        This property accepts anims.  For example, if you want to ramp
        up acceleration:

            .. sourcecode:: python

                basic_physics.a = rabbyt.lerp(0,100, dt=1000)
        """
        def __get__(self):
            return self._a
        def __set__(self, a):
            self._a = to_Anim(a)

    def update(self, dt):
        """
        ``update(dt)``

        Update the state by timestep ``dt``.

        By convention, I recommend using a timestep measured in seconds, rather
        than in milliseconds.
        """
        self._update(dt)

    cdef void _update(self, float dt):
        # TODO this needs a *real* integrator
        cdef float a, v, f

        v = self._v
        a = self._a.g()
        self._v = v + a * dt
        self._p = self._p + v*dt + .5*a*dt*dt

    cdef float g(self):
        return self._p

cdef class Basic2:
    """
    ``Basic2(xy=(0,0), vxy=(0,0), axy=(0,0), ...)``

    (You can also use the single value properties as keyword arguments, if
    you prefer.)

    This is a convenience class for using two Basic1 instances to simulate
    physics in two dimentions.

    Using item access will return the individual Basic1 instances, so this would
    work fine:

        .. sourcecode:: python

            sprite.xy = Basic2((0,10), axy=(0,-9.81))

    You can set the positions, velocities, and accelerations for x and y with
    the properties ``x``, ``y``, ``vx``, ``vy``, ``ax``, and ``ay``.

    You can also assign/retrieve two value at once with ``xy``, ``vxy``, or
    ``axy``.

    Any of these properties can be passed to ``__init__()`` as keyword
    arguments.

    ``Basic2`` instances are not automatically added to
    ``rabbyt.physics.default_update_manager``, but the individual ``Basic1``
    instances used for each dimention are.  (Adding the ``Basic2`` instance as
    well would result in the ``Basic1`` instances being updated twice!)
    """
    cdef object __weakref__

    cdef Basic1 _x, _y

    def __new__(self, *args, **kwargs):
        self._x = Basic1()
        self._y = Basic1()

    def __init__(self, xy=(0,0), vxy=(0,0), axy=(0,0), **kwargs):
        self.xy = xy
        self.vxy = vxy
        self.axy = axy
        valid_keys = "x y vx vy ax ay".split()
        for key, value in kwargs.items():
            if key not in valid_keys:
                raise TypeError("Unexpected keyword argument %r" % key)
            setattr(self, key, value)

    property x:
        """x position"""
        def __get__(self):
            return self._x.p
        def __set__(self, x):
            self._x.p = x
    property y:
        """y position"""
        def __get__(self):
            return self._y.p
        def __set__(self, y):
            self._y.p = y
    property vx:
        """x velocity"""
        def __get__(self):
            return self._x.v
        def __set__(self, x):
            self._x.v = x
    property vy:
        """y velocity"""
        def __get__(self):
            return self._y.v
        def __set__(self, y):
            self._y.v = y
    property ax:
        """x acceleration"""
        def __get__(self):
            return self._x.a
        def __set__(self, x):
            self._x.a = x
    property ay:
        """y acceleration"""
        def __get__(self):
            return self._y.a
        def __set__(self, y):
            self._y.a = y

    property xy:
        """x and y positions"""
        def __get__(self):
            return self.x, self.y
        def __set__(self, xy):
            self.x, self.y = xy
    property vxy:
        """x and y velocities"""
        def __get__(self):
            return self.vx, self.vy
        def __set__(self, vxy):
            self.vx, self.vy = vxy
    property axy:
        """x and y accelerations"""
        def __get__(self):
            return self.ax, self.ay
        def __set__(self, axy):
            self.ax, self.ay = axy

    def __len__(self):
        return 2

    def __getitem__(self, i):
        if i == 0:
            return self._x
        else:
            return self._y

    def update(self, dt):
        self._x.update(dt)
        self._y.update(dt)


class UpdateManager(object):
    """
    ``UpdateManager()``

    This is a small helper class for keeping track of physics drivers that need
    updated.

    All physics drivers are stored as weakrefs, so you don't need to worry about
    removing them.

    There is a global update manager at
    ``rabbyt.physics.default_update_manager``. All physics drivers are
    automatically added to it.
    """
    def __init__(self):
        self.updateables = []

    def append(self, updateable):
        self.updateables.append(weakref.ref(updateable, self.remove_ref))

    def remove_ref(self, reference):
        try:
            self.updateables.remove(reference)
        except ValueError:
            pass

    def update(self, float dt):
        """
        ``update(dt)``

        Update the physics drivers by timestep ``dt``.
        """
        for u in self.updateables:
            u().update(dt)

default_update_manager = UpdateManager()
update = default_update_manager.update

ctypedef struct cpVect:
    float x,y

ctypedef struct cpBody:
    float m, m_inv
    float i, i_inv

    cpVect p, v, f, v_bias
    float a, w, t, w_bias
    cpVect rot

cdef class _ChipmonkBodyX(Anim):
    cdef cpBody * body

    cdef float g(self):
        return self.body[0].p.x

cdef class _ChipmonkBodyY(Anim):
    cdef cpBody * body

    cdef float g(self):
        return self.body[0].p.y

cdef class _ChipmonkBodyA(Anim):
    cdef cpBody * body

    cdef float g(self):
        return self.body[0].a*oneeighty_over_pi


def chipmonk_body_anims(long body_addr):
    """
    ``chipmonk_body_anims(body_addr)``

    Provides fast integration with the chipmonk physics library.  Returns
    three anims for x, y, and rotation.

    ``body_addr`` Should be the address of a cpBody structure.  For example,
    if you are using the pymonk wrapper, you can do this:

        .. sourcecode:: python

            anims = chipmonk_body_anims(ctypes.addressof(body.contents))
            sprite.x, sprite.y, sprite.rot = anims

    Note that the anims store an internal pointer to the cpBody structure,
    but not a python reference.  This means that if you read the anims value
    after calling cpBodyDestroy on the body, you will get a segfault.
    """
    cdef _ChipmonkBodyX x
    x = _ChipmonkBodyX()
    x.body = <cpBody*>body_addr
    cdef _ChipmonkBodyY y
    y = _ChipmonkBodyY()
    y.body = <cpBody*>body_addr
    cdef _ChipmonkBodyA a
    a = _ChipmonkBodyA()
    a.body = <cpBody*>body_addr

    return (x,y,a)

__docs_all__ = ('Basic1 Basic2 UpdateManager update '
                'chipmonk_body_anims').split()



import rabbyt
import pygame

rabbyt.init_display((640, 480))

sprites = [rabbyt.Sprite(x=x) for x in range(-100, 100, 50)]

extend = lambda mode: rabbyt.lerp(-100, 100, 1000, 3000, extend=mode)

# Constant is the default extend mode.  It will not go beyond start or end.
sprites[0].y = extend("constant")

# With extrapolate, it just keeps going.
sprites[1].y = extend("extrapolate")

# With repeat, it starts at start again after reaching end.
sprites[2].y = extend("repeat")

# Reverse is like repeat, only every other time it moves from end to start.
sprites[3].y = extend("reverse")

while not pygame.event.peek(pygame.QUIT):
    rabbyt.clear()
    rabbyt.set_time(pygame.time.get_ticks())

    rabbyt.render_unsorted(sprites)

    pygame.display.flip()



import rabbyt
import pygame

# This is pretty much the bare minimum.

# This is a small shortcut function.  It'll call pygame.init() and
# pygame.display.set_mode() for us, and set up some default OpenGL attributes
# that rabbyt uses.
# If you want to use rabbyt with something other than pygame, you'll need to
# do these things yourself.  (Check the source, it's not hard.)
rabbyt.init_display((640, 480))

while not pygame.event.peek(pygame.QUIT):
    # Clear the screen to white.  (Colors in rabbyt are the same as in OpenGL,
    # from 0.0 to 1.0.)
    rabbyt.clear((1.0,1.0,1.0))

    pygame.display.flip()



from setuptools import setup, find_packages
from setuptools.extension import Extension

import sys

if sys.platform == "darwin":
    # On MacOS we need to include from gl.h instead of GL/gl.h, but pyrex
    # doesn't have anything along the lines of conditional compiling.  So,
    # this hack fixes up the c file after it is created.
    from Pyrex.Compiler.Main import CompilationOptions, default_options, compile
    from Pyrex.Distutils import build_ext
    def pyrex_compile(self, source):
        options = CompilationOptions(default_options,
            include_path = self.include_dirs)
        result = compile(source, options)
        if result.num_errors <> 0:
            sys.exit(1)

        if result.c_file:
            source = open(result.c_file).read()
            source = source.replace('#include "GL/gl.h"', '#include "gl.h"')
            source = source.replace('#include "GL/glu.h"', '#include "glu.h"')
            open(result.c_file, "wt").write(source)
    build_ext.pyrex_compile = pyrex_compile

if sys.platform == "win32":
    GL = "opengl32"
    GLU = "glu32"
    compile_args = []
    link_args = []
elif sys.platform == "darwin":
    compile_args=['-O3', '-framework','OpenGL','-I',
            '/System/Library/Frameworks/OpenGL.framework/Headers']
    link_args=['-dynamic','-framework','OpenGL',
            '-L/System/Library/Frameworks/OpenGL.framework/Libraries']
    GL = "GL"
    GLU = "GLU"
else:
    compile_args=['-O3']
    link_args=[]
    GL = "GL"
    GLU = "GLU"

long_description = open("README").read() + """

Changelog
=========

""" + open("CHANGELOG").read()

setup(
    name = 'Rabbyt',
    version = "0.6",
    author = "Matthew Marshall",
    author_email = "matthew@matthewmarshall.org",
    description = "A fast 2D sprite engine using OpenGL",
    license = "LGPL",
    url="http://matthewmarshall.org/projects/rabbyt/",
    long_description=long_description,

    packages = find_packages(),
    include_package_data = True,
    exclude_package_data = {'':['README', 'examples'], 'rabbyt':['*.c', '*.pyx',
            '*.pxd']},

    ext_modules=[
        Extension("rabbyt._rabbyt", ["rabbyt/rabbyt._rabbyt.pyx"],
            libraries=[GL, GLU, 'm'],
            extra_compile_args=compile_args,
            extra_link_args=link_args),
        Extension("rabbyt.anims", ["rabbyt/rabbyt.anims.pyx"],
            libraries=['m'],
            extra_compile_args=compile_args,
            extra_link_args=link_args),
        Extension("rabbyt.sprites", ["rabbyt/rabbyt.sprites.pyx"],
            libraries=[GL, 'm'],
            extra_compile_args=compile_args,
            extra_link_args=link_args),
        Extension("rabbyt.vertexarrays", ["rabbyt/rabbyt.vertexarrays.pyx"],
            libraries=[GL],
            extra_compile_args=compile_args,
            extra_link_args=link_args),
        Extension("rabbyt.collisions", ["rabbyt/rabbyt.collisions.pyx"],
            libraries=['m'],
            extra_compile_args=compile_args,
            extra_link_args=link_args),
        Extension("rabbyt.physics", ["rabbyt/rabbyt.physics.pyx"],
            libraries=['m'],
            extra_compile_args=compile_args,
            extra_link_args=link_args),
    ]
)



from __future__ import division

"""
"""

__credits__ = (
"""
Copyright (C) 2007  Matthew Marshall

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
""")

__author__ = "Matthew Marshall <matthew@matthewmarshall.org>"


import rabbyt
from rabbyt.vertexarrays import VertexArray
from rabbyt.sprites import BaseSprite

import pygame

default_alphabet = ("0123456789"
                    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                    "abcdefghijklmnopqrstuvwxyz"
                    "{}()[]<>!?.,:;'\"*&%$@#\/|+=-_~`")

def next_power_of_2(v):
    v = int(v)- 1
    v |= v >> 1
    v |= v >> 2
    v |= v >> 4
    v |= v >> 8
    v |= v >> 16
    return v + 1

class Font(object):
    """
    ``Font(pygame_font, [alphabet])``

    Holds the information for drawing a font.

    ``pygame_font`` should be a ``pygame.font.Font`` instace.

    ``alphabet`` is a string containing all of the characters that should
    be loaded into video memory.  It defaults to
    ``rabbyt.fonts.default_alphabet`` which includes numbers, symbols, and
    letters used in the English language.

    To actually draw a font you need to use a ``FontSprite``.

    The characters will be rendered onto an OpenGL texture.
    """
    def __init__(self, pygame_font, alphabet=default_alphabet):
        self.pygame_font = pygame_font
        self.alphabet = alphabet
        widest = max(self.pygame_font.size(char)[0]+1 for char in alphabet)
        height = self.pygame_font.size(" ")[1]

        total_area = widest*height*len(self.alphabet)
        sw = sh = total_area**.5
        sw += widest
        sh += height
        sw = next_power_of_2(sw)
        sh = next_power_of_2(sh)

        surface = pygame.Surface((sw, sh), pygame.SRCALPHA, 32)

        self.coords = {}
        x=0
        y=0
        for char in alphabet:
            w = self.pygame_font.size(char)[0]
            self.coords[char] = (x/sw, 1-y/sh, (x+w)/sw, 1-(y+height)/sh)
            surface.blit(
                    self.pygame_font.render(char, True, (255,255,255)), (x,y))
            x += widest
            if x + widest > sw:
                y += height
                x = 0

        data = pygame.image.tostring(surface, "RGBA", True)
        self._texture_id = rabbyt.load_texture(data, (sw, sh))
        self.height = height
        self.space_width = pygame_font.size(" ")[0]

    def _get_texture_id(self):
        return self._texture_id
    texture_id = property(_get_texture_id, doc=
        """
        This is the id of the OpenGL texture for this font.  The texture will
        be unloaded when the font is garbage collected.
        """)

    def get_char_tex_shape(self, char):
        """
        ``get_char_tex_shape(char)``

        Returns the texture shape of the given character in the form of
        ``(left, top, right, bottom)``.

        ``KeyError`` is raised if ``char`` is not in the font's ``alphabet``.
        """
        return self.coords[char]

    def get_char_width(self, char):
        """
        ``get_char_width(char)``

        Returns the width in pixels of the given character.
        """
        return self.pygame_font.size(char)[0]

    def __del__(self):
        if rabbyt:
            rabbyt.unload_texture(self.texture_id)

class FontSprite(BaseSprite):
    """
    ``FontSprite(font, text, ...)``

    A sprite that displays text from a ``Font``.

    ``font`` should be a ``rabbyt.fonts.Font`` instance to be used.

    ``text`` is the string to be displayed!  (It can be changed later with
    the text property.)

    This inherits from ``BaseSprite``, so you can use all of the
    transformation and color properties.
    """
    def __init__(self, font, text, **kwargs):
        BaseSprite.__init__(self, **kwargs)
        self.font = font
        self.texture_id = font.texture_id
        self.text = text

    def _get_text(self):
        return self._text
    def _set_text(self, text):
        self._text = text
        va = VertexArray(use_colors=False)
        h = self.font.height
        x = 0
        for i, char in enumerate(text):
            if not char in self.font.alphabet:
                x += self.font.space_width
                continue
            w = self.font.get_char_width(char)
            l, t, r, b = self.font.get_char_tex_shape(char)
            va.append((x, 0, l, t))
            va.append((x+w, 0, r, t))
            va.append((x+w, -h, r, b))
            va.append((x, -h, l, b))
            x += w+1
        va.texture_id = self.texture_id
        self.va = va
    text = property(_get_text, _set_text, doc="the text to be displayed")

    def render_after_transform(self):
        rabbyt.set_gl_color(self.rgba)
        self.va.render()

__docs_all__ = ('Font', 'FontSprite')



"""
This module provides some functions for collision detection.
"""

__credits__ = (
"""
Copyright (C) 2007  Matthew Marshall

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
""")

__author__ = "Matthew Marshall <matthew@matthewmarshall.org>"

cdef extern from "stdlib.h":
    ctypedef unsigned int size_t
    cdef void *malloc(size_t size)
    cdef void free(void *ptr)
    cdef void *realloc(void *ptr, size_t size)

    ctypedef int(*compar_func)(void *, void *)
    cdef void qsort(void *base, size_t nmemb, size_t size, compar_func compar)


def collide(objects):
    """
    ``collide(objects) -> list of collisions``

    Collides ``objects``, first using ``rdc()`` and then using
    ``brute_force()``.

    Each object should have the attributes ``x``, ``y``, ``bounding_radius``,
    and ``bounding_radius_squared``.
    """
    collisions = []
    for group in rdc(objects, min_split=10):
        if len(group) > 1:
            collisions.extend(brute_force(group))
    return collisions


cdef enum _Side:
    LEFT=1
    RIGHT=2

cdef enum _Axis:
    X=1
    Y=2

cdef struct side_s:
    float x,y
    _Side side
    int index

cdef int _compar_sides_x(void *p1, void *p2):
    cdef float x1, x2
    x1 = (<side_s**>p1)[0][0].x
    x2 = (<side_s**>p2)[0][0].x
    if x1 < x2:
        return -1
    elif x1 > x2:
        return 1
    else:
        return 0

cdef int _compar_sides_y(void *p1, void *p2):
    cdef float y1, y2
    y1 = (<side_s**>p1)[0][0].y
    y2 = (<side_s**>p2)[0][0].y
    if y1 < y2:
        return -1
    elif y1 > y2:
        return 1
    else:
        return 0

def rdc(objects, int min_split=1, int max_depth=0):
    """
    ``rdc(objects, [max_depth,] [min_split]) -> list of collision groups``

    Uses the Recursive Dimensional Clustering algorithm to find groups of
    colliding objects.

    ``objects`` should be a list of objects.  Each object should have the
    attributes ``x``, ``y``, and ``bounding_radius``.

    If the number of objects in a collision group is less than ``min_split``,
    recursion will stop.  This defaults to ``1``, but in practice, it is
    usually faster to just use a brute force method once a group gets down
    to ``10`` objects.

    ``max_depth`` is the maximum number of recursions to make.  It defaults to
    ``0``, which is infinite.

    Instead of returning individual collisions, ``rdc()`` returns groups
    (lists) of colliding objects.  For example, if ``A`` collides with ``B``
    and ``B`` collides with ``C``, one of the groups will be ``[A, B, C]``,
    even though ``A`` and ``C`` don't directly collide.

    Also, each object is returned at most once.  If it is in one group, it won't
    be in any other.  An object without any collisions isn't returned at all.
    """

    cdef side_s * side_list
    cdef side_s ** side_p_list
    cdef int length, i, d, group_start
    cdef float r, x, y
    length = len(objects)*2

    side_list = <side_s*>malloc(sizeof(side_s)*length)
    side_p_list = <side_s**>malloc(sizeof(side_s*)*length)
    try:
        for i from 0 <= i < length/2:
            o = objects[i]
            r = o.bounding_radius
            x = o.x
            y = o.y
            side_list[i*2].x = x-r
            side_list[i*2].y = y-r
            side_list[i*2].side = LEFT
            side_list[i*2].index = i
            side_list[i*2+1].x = x+r
            side_list[i*2+1].y = y+r
            side_list[i*2+1].side = RIGHT
            side_list[i*2+1].index = i

            side_p_list[i*2] = &side_list[i*2]
            side_p_list[i*2+1] = &side_list[i*2+1]

        _rdc(side_p_list, length, X, 0, min_split, max_depth)

        groups = []
        current_group = []
        d = 0
        for i from 0 <= i < length:
            if side_p_list[i][0].side == LEFT:
                if d == 0:
                    # Check to see if we are a single object without any
                    # collisions.  If so, we shouldn't be added as a group.
                    # (Doing this can cut running time in half.)
                    if side_p_list[i][0].index == side_p_list[i+1][0].index:
                        # The next item will be the right side of the same
                        # object; we can safely skip it.
                        i = i + 1
                        continue
                d = d + 1
                current_group.append(objects[side_p_list[i][0].index])
            else:
                d = d - 1
                if d == 0:
                    groups.append(current_group)
                    current_group = []
        return groups
    finally:
        free(side_list)
        free(side_p_list)

cdef void _rdc(side_s ** side_p_list, int length, _Axis axis, int depth,
        int min_split, int max_depth):
    cdef int i
    if length <= min_split*2:
        return
    if max_depth > 0 and depth >= max_depth:
        return

    cdef _Axis next_axis

    if axis == X:
        qsort(side_p_list, length, sizeof(side_s*), _compar_sides_x)
        next_axis = Y
    else:
        qsort(side_p_list, length, sizeof(side_s*), _compar_sides_y)
        next_axis = X

    cdef int group_start, d

    group_start = 0
    d = 0
    for i from 0 <= i < length:
        if group_start == 0 and i == length-1:
            # We only have one group. If we are the first call, go ahead and try
            # the Y access.  Otherwise, let's just bail out now.
            if depth == 0:
                _rdc(side_p_list, length, Y, 1, min_split, max_depth)
            return

        if side_p_list[i][0].side == LEFT:
            d = d + 1
        else:
            d = d - 1
            if d == 0:
                _rdc(&side_p_list[group_start], i-group_start+1, next_axis,
                        depth+1, min_split, max_depth)
                group_start = i + 1



cdef struct collision_object_s:
    float x, y, brs

cdef _brute_force(collision_object_s * objs, int length, objects):
    cdef float dx, dy
    cdef int i, j
    collisions = []
    for i from 0 <= i < length-1:
        for j from i < j < length:
            dx = objs[i].x - objs[j].x
            dy = objs[i].y - objs[j].y
            if dx*dx + dy*dy < objs[i].brs + objs[j].brs:
                collisions.append((objects[i],
                        objects[j]))
    return collisions

def brute_force(objects):
    """
    ``brute_force(objects) -> list of collisions``

    Finds collisions between ``objects`` using a brute force algorithm.

    ``objects`` should be a list of objects, each of which have the attributes
    ``x``, ``y``, and ``bounding_radius_squared``.  Each object is checked
    against every other object.

    For example, if ``A`` collides with ``B``, ``B`` collides with ``C``, and
    ``D`` doesn't collide with anything, the result will be:
    ``[(A, B), (B, C)]``.
    """
    cdef collision_object_s * objs
    cdef int i, length

    length = len(objects)
    objs = <collision_object_s*>malloc(sizeof(collision_object_s)*length)
    try:
        # First we move the data from the python objects into c structures.
        # This is especially important as most of the objects will be sprites,
        # and accessing their positions could set off a long chain of
        # calculations.  We only want to do that once.
        for i from 0 <= i < length:
            o = objects[i]
            objs[i].x = o.x
            objs[i].y = o.y
            objs[i].brs = o.bounding_radius_squared
        # Do the actual work:
        return _brute_force(objs, length, objects)
    finally:
        free(objs)

__docs_all__ = 'collide rdc brute_force'.split()


import rabbyt
import pygame

import os.path
rabbyt.data_directory = os.path.dirname(__file__)


rabbyt.init_display((640, 480))

car = rabbyt.Sprite("car.png")

# You don't have to be limited to static values!  Using rabbyt.lerp you can
# assign a value that linearly interpolates over time.  The interpolation is
# done in Pyrex code, so it is practically free.

# Fade the car in after one second.
car.alpha = rabbyt.lerp(0.0, 1.0, startt=1000, endt=2000)

# Rotate the car from 0 to 360 over three seconds, then repeat.
car.rot = rabbyt.lerp(0, 360, dt=3000, extend="repeat")

while not pygame.event.peek(pygame.QUIT):
    rabbyt.clear((1,1,1))

    # When using anims (such as returned by lerp,) we need to tell
    # rabbyt what time it is every frame.  This might sound silly, but there
    # are two good reasons for it.  1) Rabbyt isn't limited to one
    # high-resolution clock implementaion and 2) you can have complete control
    # over time.  This could be very useful if you want to add a 'pause' feature
    # to you game :)
    rabbyt.set_time(pygame.time.get_ticks())

    car.render()

    pygame.display.flip()



from __future__ import division

import random

import rabbyt, rabbyt.collisions, rabbyt.physics
from rabbyt import lerp, wrap

import pygame

rabbyt.init_display((800, 600))


sprites = []

r = lambda: random.random()-.5

class BumperCraft(rabbyt.Sprite):
    def __init__(self, x, y):
        rabbyt.Sprite.__init__(self)

        self._phy_x = rabbyt.physics.Basic1()
        self._phy_y = rabbyt.physics.Basic1()

        self._phy_x.p = x
        self._phy_y.p = y

        self.x = wrap([-400,400], lambda: self._phy_x.p)
        self.y = wrap([-300,300], lambda: self._phy_y.p)

        self._phy_x.a = lambda: 100 #self.convert_offset((10,0))[0]
        self._phy_y.a = lambda: 100 #self.convert_offset((10,0))[1]

    def update(self, dt):
        self._phy_x.update(dt)
        self._phy_y.update(dt)


for i in range(400):
    b = BumperCraft(r()*800, r()*600)

    sprites.append(b)

c = pygame.time.Clock()
while not pygame.event.peek(pygame.QUIT):
    dt = c.tick(60)

    rabbyt.set_time(pygame.time.get_ticks())

    time = dt/1000 # Update will expect time in seconds, not ms.
    for b in sprites:
        b.update(time)

    rabbyt.clear()

    collisions = rabbyt.collisions.brute_force(sprites)

    for s1, s2 in collisions:
        s1.rgb = lerp((1,0,0),(1,1,1), dt=400)
        s2.rgb = lerp((1,0,0),(1,1,1), dt=400)

    rabbyt.render_unsorted(sprites)
    pygame.display.flip()



__credits__ = (
"""
Copyright (C) 2007  Matthew Marshall

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
""")

__author__ = "Matthew Marshall <matthew@matthewmarshall.org>"



cdef extern from "math.h":
    cdef float fmodf(float x, float y)
    cdef float cosf(float x)
    cdef float sinf(float x)
    cdef float sqrtf(float x)

cdef extern from "GL/gl.h":
    ctypedef float GLfloat
    ctypedef float GLclampf
    ctypedef unsigned int GLenum
    ctypedef unsigned int GLbitfield
    ctypedef int GLint
    ctypedef unsigned int GLuint
    ctypedef int GLsizei
    ctypedef double GLdouble
    ctypedef double GLclampd
    ctypedef void GLvoid
    ctypedef unsigned char GLubyte

    cdef int GL_SMOOTH
    cdef int GL_COLOR_BUFFER_BIT
    cdef int GL_BLEND
    cdef int GL_SRC_ALPHA
    cdef int GL_ONE_MINUS_SRC_ALPHA
    cdef int GL_TEXTURE_2D
    cdef int GL_QUADS
    cdef int GL_MODELVIEW
    cdef int GL_RGBA
    cdef int GL_RGB
    cdef int GL_NEAREST
    cdef int GL_LINEAR
    cdef int GL_TEXTURE_MAG_FILTER
    cdef int GL_TEXTURE_MIN_FILTER
    cdef int GL_TEXTURE_ENV
    cdef int GL_TEXTURE_ENV_MODE
    cdef int GL_MODULATE
    cdef int GL_LINEAR_MIPMAP_NEAREST
    cdef int GL_UNSIGNED_BYTE
    cdef int GL_PROJECTION
    cdef int GL_FLAT
    cdef int GL_FLOAT
    cdef int GL_POLYGON_SMOOTH

    cdef int GL_VENDOR, GL_RENDERER, GL_VERSION, GL_EXTENSIONS

    cdef void glTranslatef(GLfloat x, GLfloat y, GLfloat z)
    cdef void glEnable(GLenum cap)
    cdef void glDisable(GLenum cap)
    cdef void glClear(GLbitfield mask)
    cdef void glShadeModel(GLenum mode)
    cdef void glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
    cdef void glBlendFunc(GLenum sfactor, GLenum dfactor)
    cdef void glBindTexture(GLenum target, GLuint texture)
    cdef void glColor4f(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha)
    cdef void glBegin(GLenum mode)
    cdef void glTexCoord2f(GLfloat s, GLfloat t)
    cdef void glVertex2f(GLfloat x, GLfloat y)
    cdef void glEnd()
    cdef void glViewport(GLint x, GLint y, GLsizei width, GLsizei height)
    cdef void glOrtho(GLint bottom, GLint left, GLint top, GLint bottom, GLint front, GLint back)
    cdef void glMatrixMode(GLenum mode)
    cdef void glLoadIdentity()
    cdef void glTexParameteri(GLenum target, GLenum pname, GLint param)
    cdef void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, GLvoid *pixels)
    cdef void glGenTextures(GLsizei n, GLuint *textures)
    cdef void glDeleteTextures(GLsizei n, GLuint *textures)
    cdef void glTexEnvf(GLenum target, GLenum pname, GLfloat param)

    cdef GLubyte *glGetString(GLenum name)

cdef extern from "GL/glu.h":
    cdef GLint gluBuild2DMipmaps( GLenum target, GLint internalFormat, GLsizei width, GLsizei height, GLenum format, GLenum type, void *data)


load_texture_file_hook = None
def set_load_texture_file_hook(callback):
    """
    ``set_load_texture_file_hook(callback)``

    This sets the ``callback`` that is used to load a texture from a file.
    (It is called when you give a string for the first argument of
    ``Sprite``.)

    The default is ``rabbyt.pygame_load_texture``, which will use pygame to
    load the file.  You may write your own callback if you don't want to use
    pygame.

    The ``callback`` should take the filename as a single argument, and return
    a tuple of the form ``(texture_id, (width, height))``.
    """
    global load_texture_file_hook
    load_texture_file_hook = callback



def render_unsorted(sprites):
    """
    ``render_unsorted(sprites)``

    Renders a list of sprites.

    Since this function is implemented in Pyrex, it should be a little faster
    than looping through the sprites in Python.
    """
    for s in sprites:
        s.render()


def render_sorted(sprites):
    ss = list(sprites)
    ss.sort()
    render_unsorted(ss)


def set_viewport(viewport, projection=None):
    """
    ``set_viewport(viewport, [projection])``

    Sets how coordinates map to the screen.

    ``viewport`` gives the screen coordinates that will be drawn to.  It
    should be in either the form ``(width, height)`` or
    ``(left, top, right, bottom)``

    ``projection`` gives the sprite coordinates that will be mapped to the
    screen coordinates given by ``viewport``.  It too should be in one of the
    two forms accepted by ``viewport``.  If ``projection`` is not given, it
    will default to the width and height of ``viewport``.  If only the width
    and height are given, ``(0, 0)`` will be the center point.
    """
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    if len(viewport) == 4:
        l, t, r, b = viewport
    else:
        l, t = 0, 0
        r, b = viewport
    glViewport(l, t, r-l, b-t)

    if projection is not None:
        if len(projection) == 4:
            l, t, r, b = projection
        else:
            w,h = projection
            l, r, t, b = -w/2, w/2, -h/2, h/2
    else:
        w,h = r-l, b-t
        l, r, b, t = -w/2, w/2, -h/2, h/2
    glOrtho(l, r, b, t, -1, 1)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def set_default_attribs():
    """
    ``set_default_attribs()``

    Sets a few of the OpenGL attributes that sprites expect.

    Unless you know what you are doing, you should call this at least once
    before rendering any sprites.  (It is called automatically in
    ``rabbyt.init_display()``)
    """
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE)
    glEnable(GL_BLEND)
    #glEnable(GL_POLYGON_SMOOTH)


def set_gl_color(rgba):
    """
    ``set_gl_color(rgba)``

    This is a thin wrapper around ``glColor4f()``.
    """
    cdef float r,g,b,a
    r, g, b, a = rgba
    glColor4f(r,g,b,a)

def get_gl_vendor():
    """
    ``gl_get_vendor()``

    Returns the OpenGL vendor string.  Returns None if there is no context.
    """
    cdef char * string
    string = <char*>glGetString(GL_VENDOR)
    if <int>string == 0:
        return None
    else:
        return string


# The following functions were ripped from Simmon Wittber's GFX library:

def load_texture(byte_string, size, type='RGBA', filter=True, mipmap=True):
    """
    ``load_texture(byte_string, size, type='RGBA', filter=True, mipmap=True)``

    Load a texture and return it.
    """
    cdef GLuint textures[1]
    cdef GLuint id
    glGenTextures(1, textures)
    id = textures[0]
    update_texture(id, byte_string, size, type, filter, mipmap)
    return id

def update_texture(texture_id, byte_string, size, type='RGBA', filter=True,
        mipmap=True):
    """
    ``update_texture(texture_id, byte_string, size, type='RGBA', filter=True,
    mipmap=True)``

    Update a texture with a different byte_string.
    """

    if not get_gl_vendor():
        raise RuntimeError("Trying to load a texture without an OpenGL context")

    cdef char *data
    data = byte_string
    if type == 'RGBA':
        ptype = GL_RGBA
        channels = 4
    elif type == 'RGB':
        ptype = GL_RGB
        channels = 3
    else:
        raise ValueError('type must be "RGBA" or "RGB"')

    if size[0]*size[1]*channels != len(byte_string):
        raise ValueError('byte_string is an unexpected size.')

    filter_type = GL_NEAREST
    if filter: filter_type = GL_LINEAR
    glBindTexture(GL_TEXTURE_2D, texture_id)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter_type)
    if mipmap:
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST)
        gluBuild2DMipmaps(GL_TEXTURE_2D, channels, size[0], size[1], ptype, GL_UNSIGNED_BYTE, data)
    else:
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter_type)
        glTexImage2D(GL_TEXTURE_2D, 0, ptype, size[0], size[1], 0, ptype, GL_UNSIGNED_BYTE, data)

def unload_texture(texture_id):
    """
    ``unload_texture(texture_id)``

    Unload a texture from memory.
    """
    cdef GLuint textures[1]
    textures[0] = texture_id
    glDeleteTextures(1, textures)

def clear(rgba=(0.0,0.0,0.0,1.0)):
    """
    ``clear(rgba=(0.0,0.0,0.0,1.0))``

    Clear the screen to a background color.
    """
    if len(rgba) == 3:
        rgba = tuple(rgba) + (1.0,)
    glClearColor(rgba[0], rgba[1], rgba[2], rgba[3])
    glClear(GL_COLOR_BUFFER_BIT)
